function varargout = get_aviso_altimetry(varargin)
% Gets AVISO Altimetry data.
%
% get_aviso_altimetry
% get_aviso_altimetry(request)
%
% INPUT(S):
%
%   request -- (optional) structure input which contains
%              information necessary to obtain data. The GUI
%              merely provides a graphical method of generating
%              the "request" variable. However, this visual
%              interface may be by-passed by this method.
%
% OUTPUT(S):
%
%   [none]
%
% OPeNDAP Science Team
% Copyright 2008
% $Version 1.0.0$

%==========================================================================
% Meri Sheremet
% April 2008
%
% REVISION HISTORY:
% 2008/04/10 created
% 2008/04/24 working version
% 2008/07/02 update dsi file only if not today's date on file
%==========================================================================

if ~exist('loaddap')
    errordlg('loaddap not found. Make sure it is in your MATLAB path.');
    return
end               

if nargin<1
% no arguments passed, get parameters from GUI 

dsi_filename = 'dsi_aviso_altimetry.mat';
if (~isdeployed)
    pname = mfilename('fullpath');
    pname = pname(1:length(pname) - length(mfilename));
    dsi_filename = [pname,dsi_filename];
end

dsi_flag = 0;

if exist(dsi_filename,'file')
    
    % Get date of file.
    dum = dir(dsi_filename);
    
    if floor(now)>datenum(dum.date) %if today's date is different than that of file
        
        msgtxt = ['Acquiring metadata from AVISO site. ',...
             'Updating available time range. ',...
             'Please wait ...'];
        dsi_flag = 1;
        
    end
    
else %does not exist
    
    msgtxt = ['Acquiring metadata from AVISO site. ',...
         'Please wait ...'];
    dsi_flag = 1;
    
end

if dsi_flag==1
    
    hbox=msgbox(msgtxt);
        % next line makes OK button invisible
    hh=get(hbox,'Children');
    set(hh(2),'Visible','off');
    pause(1);
    
    % generate dsi file.
    dsi_aviso_altimetry
end

h = gui_aviso_altimetry;
if nargout
    varargout{1} = h; %set figure handle as output
end
elseif nargin > 1
disp('Incorrect number of arguments.'); return  
else
% one argument request, get AVISO Altimetry data  
request=varargin{1}; 
request.RequestDate=datestr(now,'yyyy-mm-dd HH:MM:SS');
R=request;

% load DataSet Inventory and Interface Structure
%example: R.DataSetBranch='Merged_Product_Delayed_Time_NG';
% load dsi_OceanColor.mat DSI_Merged_Product_Delayed_Time_NG
eval(['load dsi_aviso_altimetry.mat DSI_' R.DataSetBranch ';'])
eval(['DSI=DSI_' R.DataSetBranch ';'])
eval(['clear DSI_' R.DataSetBranch ';'])

%assignin('base','request',request);
%assignin('base','DSI',DSI);

Time=DSI.Time;
TimeA=DSI.TimeA;
TimeB=DSI.TimeB;
Latitude=DSI.Latitude;
Longitude=DSI.Longitude;

NT=length(Time);
NY=length(Latitude);
NX=length(Longitude);
NV=length(DSI.Fields(:,1));

if(isempty(R.DATE1) | isempty(R.DATE2)) | (str2num(R.DATE2)<str2num(R.DATE1))
    errordlg(['Selected dates are invalid.']);
    return
end
%JD1=datenum(R.DATE1,'yyyymmdd')
%JD2=datenum(R.DATE2,'yyyymmdd')+1;
%assignin('base','Time',Time)
%iDATE1=min(find(TimeB > JD1));
%iDATE2=max(find(TimeA < JD2));
JD1=datenum(R.DATE1,'yyyymmdd');
JD2=datenum(R.DATE2,'yyyymmdd')+1;
%assignin('base','Time',Time)
%iDATE1=max(find(TimeA < JD1));
%iDATE2=min(find(TimeB > JD2));
iDATE1=min(find(Time >= JD1));
iDATE2=max(find(Time <  JD2));
if(isempty(iDATE1) | isempty(iDATE2)) | (iDATE2<iDATE1)
    errordlg(['No data within the requested date range.']);
    return
end

if isempty(R.LAT1) | isempty(R.LAT2) | isempty(R.LON1) | isempty(R.LON2)
    errordlg(['Latitude/Longitude values are invalid or empty.']);
    return
end

% adjust entered longitudes modulo 360 to match the dataset range 
LonM=0.5*(Longitude(1)+Longitude(end));
R.LON1=R.LON1+floor((LonM+180-R.LON1)/360)*360;
R.LON2=R.LON2+floor((LonM+180-R.LON2)/360)*360;

% Find indices within the specified range of lons, lats
iLAT1=min(find(Latitude >= R.LAT1))-1;
iLAT2=max(find(Latitude <= R.LAT2))-1;
iLON1=min(find(Longitude >= R.LON1))-1;
iLON2=max(find(Longitude <= R.LON2))-1;
if(isempty(iLAT1) | isempty(iLAT2)) | (iLAT2<iLAT1)
    errordlg(['Requested latitudes are out of range.']);
    return
end
if(isempty(iLON1) | isempty(iLON2)) | (iLON2<iLON1)
    errordlg(['Requested longitudes are out of range.']);
    return
end
if(isempty(R.LATINCR)|isempty(R.LONINCR)) 
    errordlg(['Subsampling values are invalid.']);
    return
end

if R.SaveWorkspace=='n' & R.SaveFiles=='n'
    errordlg(['Select how to save data.']);
    return
end
if length(R.DIRNAME) > 0
    if exist(R.DIRNAME)==0
        eval(['mkdir ' R.DIRNAME])
    end    
end

CLAT=['[' num2str(iLAT1) ':' num2str(R.LATINCR) ':' num2str(iLAT2) ']']; % ConstraintLatitude
CLON=['[' num2str(iLON1) ':' num2str(R.LONINCR) ':' num2str(iLON2) ']']; % ConstraintLongitude

% Estimate the amount of requested data
Nt=length(iDATE1:R.DATEINCR:iDATE2);
Ny=length(iLAT1:R.LATINCR:iLAT2);
Nx=length(iLON1:R.LONINCR:iLON2);
Nv=0;
for k=1:length(DSI.Fields(:,1))
    if R.Fields{k,end}=='y' Nv=Nv+1; end % number of checked fields
end
if Nv==0
    errordlg(['No variables have been selected.']);
    return
end
MB=round(Nt*Ny*Nx*Nv*8/1024/1024);
MBLIMIT=100; % warning if requested amount of data > MBLIMIT
if MB>MBLIMIT
ButtonName=questdlg(['The amount of data requested is ' num2str(MB) ' MB.'], ...
        'Requested Data Amount','Continue','Cancel','Cancel');
    if strcmp(ButtonName,'Cancel') return; end
end

if strcmp(R.SaveMode,'merged')==0 
% find the latest saved opendap or netcdf frame number
[kw0,kf0]=op_lastframenum(R);
end


% ***************************************************
% alternative saving mode for user convenience
% save time sequence of frames as a single 3D array 
% warning 3D array may not fit into computer memory
if strcmp(R.SaveMode,'merged')==1
clear D3
    for k=1:NV
        if R.Fields{k,end}=='y';
% select only checked variables
FIELD=R.Fields{k,1};
eval(['D3.' FIELD '=zeros(Nt,Ny,Nx)*NaN;']);
        end
    end
D3.time=zeros(Nt,1)*NaN;
end
% ********************************************************************

hwaitbar=waitbar(0,'Acquiring ','Name','AVISO Download Progress');
kf=0;
%URL0=[DSI.URLSITE DSI.URLPATH DSI.URLFILE];
%A=loaddap('-A',URL0); 

for k=iDATE1:R.DATEINCR:iDATE2 % loop through dates
DATE=datestr(Time(k),'yyyymmdd');
kf=kf+1;

clear D
% create standard blank opendap_nnnn data structure
for kv=1:NV
        if R.Fields{kv,end}=='y'    
FIELD=R.Fields{kv,1}; %
eval(['D.' FIELD '=NaN;']);
        end
end
% lat,lon,time are supposed to be same for all fields
D.latitude=Latitude(iLAT1+1:R.LATINCR:iLAT2+1);
D.longitude=Longitude(iLON1+1:R.LONINCR:iLON2+1);
%D.time=Time(k); % datenum;  
D.time=(Time(k)-datenum(1970,1,1,0,0,0))*86400;% in seconds since 1970-01-01 00:00:00

for kv=1:NV
        if R.Fields{kv,end}=='y'    
FIELD=R.Fields{kv,1}; %
eval(['D.metadata.' FIELD '_variables=' 'DSI.Fields_Coord{kv};']);
        end
end
D.metadata.url='';
D.metadata.attributes='';
D.metadata.name=lower(DSI.DataSetBranch);
D.metadata.temporal='time_series';
D.metadata.reference_time='1970-01-01 00:00:00';
D.metadata.readme=op_format_readme(DSI.Readme);
D.metadata.user_friendly_time=datestr(Time(k),'yyyy-mm-dd HH:MM:SS'); % Gregorian
%D.metadata.timeGregorian=NaN;
%D.metadata.time1970seconds=NaN;
D.metadata.request_date=R.RequestDate;
D.metadata.request=R;

kVV=0;
    for kV=1:NV % loop through checked variables
        if R.Fields{kV,end}=='y'
            
            % a general request employed by get_DataSet program has the following pattern
% loaddap([URLSITE URLPATH URLFILE '?' URLCVAR URLCTIME URLCDEPTH URLCLAT URLCLON ])
%   constraints URLCTIME URLCDEPTH URLCLAT URLCLON  can be in different order
%   constraints URLCLAT URLCLON URLCDEPTH will be generated by program get

FIELD=R.Fields{kV,1};
URLCVAR=DSI.Fields_NameDS{kV};
FIELDR=[URLCVAR '.' URLCVAR];
%VariableDim=DSI.Fields_DimDS{kV};
kVV=kVV+1;

xprogr=((kVV-1)/Nv+(kf-1))/Nt; % progress fraction
s= strrep(FIELD,'_',' ');
waitbar( xprogr , hwaitbar, ['Acquiring ' s ' ' DATE],'Name','AVISO Altimetry Download Progress');

%C0='[0:1:0]'; % constraint time 
C1=['[' num2str(k-1) ':1:' num2str(k-1) ']']; % Constraint Time
C3=['[' num2str(iLAT1) ':' num2str(R.LATINCR) ':' num2str(iLAT2) ']']; % Constraint Latitude
C2=['[' num2str(iLON1) ':' num2str(R.LONINCR) ':' num2str(iLON2) ']']; % Constraint Longitude

URL0=[DSI.URLSITE DSI.URLPATH DSI.URLFILE{kV}];
URL=[URL0 '?' URLCVAR C1 C2 C3 ];    % 3D variables
disp(URL)

loaddap(['+v','-e'],URL);
%dods_err = 0 means no error.
%dods_err = 1 means error.
%dods_err_msg [variable where error message is stored]
if dods_err
    disp(dods_err_msg)
    msgbox('Could not access dataset site. Sorry, try again later.');
    close(hwaitbar);
    return
end
A=loaddap(['-A','+v'],URL); 

% current field is called f
eval(['f=squeeze(' FIELDR ');']);

% replace missing values with NaN according to formula stored in DSI.FormulaNaN
%eval(eval(['DSI.FormulaNaN.' FIELD]))
eval(eval(['DSI.FormulaNaN'])); % same for all fields
% convert to physical values according to formula stored in DSI.FormulaCnv
%eval(eval(['DSI.FormulaCnv.' FIELD]))

% transpose matrix to have f(lat,lon)
eval(eval(['DSI.FormulaOrderCoord']));
%f=f';

eval(['D.' FIELD '=f;']);
eval(['D.metadata.url.' FIELD '=URL;']);
eval(['D.metadata.attributes.' FIELD '=A;']); 

%if R.SaveWorkspace == 'y'& strcmp(R.SaveMode,'opendap')==0
%assignin('base',[FIELD '_' DATE],f)
%end
%if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==0
%% save extracted data to files
%FN3=[R.FNPREFIX FIELD '_' DATE '.mat'];
%    if length(R.DIRNAME) > 0
%    FN3=[R.DIRNAME '/' FN3];
%    else
%    FN3=[pwd '/' FN3];
%    end
%FileName=FN3
%disp(['    saving data to file: ' FN3])
%save(FN3,FIELD,'URL')
%end

        end % if checked variables
    end % kV list of variables

if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'merged')==0
KF=num2str(kf+kw0,'%04d');
WSN=['opendap_' KF];
assignin('base',WSN,D)
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==1
% save extracted data to files
KF=num2str(kf+kf0,'%04d');
FN3=[R.FNPREFIX KF '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    else
    FN3=[pwd '/' FN3];
    end
FileName=FN3;
disp(['    saving data to file: ' FN3])
save(FN3,'D')
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'netcdf')==1
% save extracted data to files
KF=num2str(kf+kf0,'%04d');
FN3=[R.FNPREFIX KF '.nc'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    else
    FN3=[pwd '/' FN3];
    end
FileName=FN3;
op_writenetcdf(D,FN3);
end


if strcmp(R.SaveMode,'merged')==1
    for kv=1:NV
        if R.Fields{kv,end}=='y'
            FIELD=R.Fields{kv,1};
eval(['D3.' FIELD '(kf,:,:)=D.' FIELD ';']);
        end
    end
    D3.time(kf)=Time(k);
end

end % loop through dates

% *************************************************************
% alternative saving mode for user convenience
% save as a single 3D array 
% warning 3D array may not fit into computer memory
if strcmp(R.SaveMode,'merged')==1
    D3.latitude=D.latitude;
    D3.longitude=D.longitude;
    
     %   D3.timeGregorian=Time(iTime,:);
    D3.metadata.coordinates={'time','latitude','longitude'};
    D3.metadata.name=DSI.DataSetBranch;
    D3.metadata.request=R;
end
if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'merged')==1
assignin('base','aviso_altimetry',D3)
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'merged')==1
% save extracted data to files
FN3=[R.FNPREFIX 'aviso_altimetry' '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    else
    FN3=[pwd '/' FN3];
    end
FileName=FN3;
disp(['    saving data to file: ' FN3])
save(FN3,'D3')
end

xprogr=1;
s=strrep(FIELD,'_',' ');
waitbar( xprogr , hwaitbar, ['Acquiring ' s ' ' DATE],'Name','AVISO Altimetry Download Progress');
close(hwaitbar)
%msgbox('The data have been acquired.')
disp('The data have been acquired.')
end % nargin

