function varargout = get_modis(varargin)
% Gets Moderate Resolution Imaging Spectroradiometer (MODIS) data.
% from NASA/JPL.
%
% get_modis
% get_modis(request)
%
% INPUT(S):
%
%   request -- (optional) structure input which contains
%              information necessary to obtain data. The GUI
%              merely provides a graphical method of generating
%              the "request" variable. However, this visual
%              interface may be by-passed by this method.
%
% OUTPUT(S):
%
%   [none]
%
% OPeNDAP Science Team
% Copyright 2007
% $Version 1.00$

%==========================================================================
% Meri Sheremet
% May 2007
%
% REVISION HISTORY:
% 2007/01/01 created, ms
%==========================================================================

URLBase='http://dods.jpl.nasa.gov/dods-bin/nph-hdf/pub/sea_surface_temperature/modis/data';
MBLIMIT=100;

if nargin<1
% no arguments passed, get parameters from GUI     
h = gui_modis;
if nargout
    varargout{1} = h; %set figure handle as output
end
elseif nargin > 1
disp('Incorrect number of arguments.'); return  
else
% one argument request, get MODIS data  

request=varargin{1};
R=request;
DATE1 = R.DATE1;
DATE2 = R.DATE2;
DATEINCR = R.DATEINCR;
LAT1 = R.LAT1;
LAT2 = R.LAT2;
LATINCR = R.LATINCR;
LON1 = R.LON1;
LON2 = R.LON2;
LONINCR = R.LONINCR;
Fields = R.Fields;
Platform=R.Platform;
Pass = R.Pass;
TAVG = R.TAVG;
HRES=R.HRES;
ApplyLandMask=R.ApplyLandMask;
ApplyQualMask=R.ApplyQualMask;
CalcAnomaly=R.CalcAnomaly;
SaveWorkspace = R.SaveWorkspace;
SaveWorkspaceMode = R.SaveWorkspaceMode;
SaveFiles = R.SaveFiles;
DIRNAME = R.DIRNAME;
FNPREFIX = R.FNPREFIX;

% Define parts of request structure.
% load('PointerToMainFunction.mat','main_function');
% guiname = mfilename;
% request.function = guiname;
% request.version = op_version(main_function);
% assignin('base','request',request);

if(~exist('loaddap','file'))
    errordlg('loaddap not found. Make sure it is in your MATLAB path.')
    return
end
       
if isempty(LAT1) | isempty(LAT2) | isempty(LON1) | isempty(LON2)
    errordlg('Latitude/Longitude values are invalid or empty. Reissue Get_MODIS command and correct the problem.')
    return
end

% theoretical values
d=360/8640; %8192; %04km%
if strcmp(HRES,'36km') d=360/1080; %1024; 
end % actually at the equator 40000/1024=39.0625 km
if strcmp(HRES,'1deg') d=360/360; 
end
Latitude=(90-d/2:-d:-90+d/2)';
Longitude=(-180+d/2:d:180-d/2)';

iLAT2=max(find(Latitude >= LAT1))-1;
iLAT1=min(find(Latitude <= LAT2))-1;
iLON1=min(find(Longitude >= LON1))-1;
iLON2=max(find(Longitude <= LON2))-1;

if(isempty(iLAT1) | isempty(iLAT2)) | (iLAT2<iLAT1)
    errordlg('Requested latitudes are out of range. Reissue Get_MODIS command and correct selected latitudes.')
    return
end
if(isempty(iLON1) | isempty(iLON2)) | (iLON2<iLON1)
    errordlg('Requested longitudes are out of range. Reissue Get_MODIS command and correct selected longitudes.')
    return
end
if(isempty(LATINCR)|isempty(LONINCR)) 
    errordlg('Subsampling values are invalid. Reissue Get_MODIS command and correct the problem.')
    return
end
if(isempty(DATE1) | isempty(DATE2)) | (str2num(DATE2)<str2num(DATE1))
    errordlg('Selected dates are invalid. Reissue Get_MODIS command and correct the problem.')
    return
end

if SaveWorkspace=='n' & SaveFiles=='n'
    errordlg('Select how to save data. Reissue Get_MODIS command and correct the problem.')
    return
end

if length(DIRNAME) > 0
    if exist(DIRNAME)==0
        eval(['mkdir ' DIRNAME])
    end    
end

C1=['[' num2str(iLAT1) ':' num2str(LATINCR) ':' num2str(iLAT2) ']']; % ConstraintLatitude
C2=['[' num2str(iLON1) ':' num2str(LONINCR) ':' num2str(iLON2) ']']; % ConstraintLongitude

% designate TAVGBYTE for saving data
if     strcmp(TAVG,'daily')   TAVGBYTE='d';
elseif strcmp(TAVG,'weekly')  TAVGBYTE='w';
elseif strcmp(TAVG,'monthly') TAVGBYTE='m';
else                          TAVGBYTE='x'; % not used
end

% testing
%URL_StdDev=[URL_CatBase '/daily/04km/daily-04km-StdDev.dat' '?' CPLATFORM CDATE];
%URL_Quality=[URL_CatBase '/daily/04km/daily-04km-Quality.dat' '?' CPLATFORM CDATE];
%URL_Number=[URL_CatBase '/daily/04km/daily-04km-Number.dat' '?' CPLATFORM CDATE];
            if 0
URL_SST=Get_MODIS_URLs(DATE1,DATE2,DATEINCR,Platform,TAVG,HRES,Pass,'SST');
URL_StdDev=Get_MODIS_URLs(DATE1,DATE2,DATEINCR,Platform,TAVG,HRES,Pass,'StdDev');
URL_Quality=Get_MODIS_URLs(DATE1,DATE2,DATEINCR,Platform,TAVG,HRES,Pass,'Quality');
URL_Number=Get_MODIS_URLs(DATE1,DATE2,DATEINCR,Platform,TAVG,HRES,Pass,'Number');
URL_Flag1=Get_MODIS_URLs(DATE1,DATE2,DATEINCR,Platform,TAVG,HRES,Pass,'Flag1');
URL_Flag2=Get_MODIS_URLs(DATE1,DATE2,DATEINCR,Platform,TAVG,HRES,Pass,'Flag2');
URL_ComFlags=Get_MODIS_URLs(DATE1,DATE2,DATEINCR,Platform,TAVG,HRES,Pass,'ComFlags');
assignin('base',['URL_SST'],URL_SST) 
assignin('base',['URL_StdDev'],URL_StdDev) 
assignin('base',['URL_Quality'],URL_Quality) 
assignin('base',['URL_Number'],URL_Number) 
assignin('base',['URL_Flag1'],URL_Flag1) 
assignin('base',['URL_Flag2'],URL_Flag2) 
assignin('base',['URL_ComFlags'],URL_ComFlags) 
save MODIS_Catalog URL_SST URL_StdDev URL_Quality URL_Number URL_Flag1 URL_Flag2 URL_ComFlags
return
            end

            % temporary only sst and qual work
%Fields={'sst','n';'sst4','n';'qual','n';'sdev','n';'num','n';'flag1','n';'flag2','n';'cflag','n'};
Fields{2,2}='n';
Fields{4,2}='n';
Fields{5,2}='n';
Fields{6,2}='n';
Fields{7,2}='n';
Fields{8,2}='n';
TAVG='daily';
HRES='04km';

            

NU=0; % total number of requested URLs
[L1,L2]=size(Fields);
for kV=1:L1 % loop through checked variables
    if Fields{kV,2}=='y'
VariableName=Fields{kV,1};

CSVARNAME=''; % name of the variable in the catalog server
%Fields={'sst','n';'sst4','n';'qual','n';'sdev','n';'num','n';'flag1','n';'flag2','n';'cflag','n'};
if strcmp(VariableName,'sst')   CSVARNAME='SST'; end
if strcmp(VariableName,'sst4')  CSVARNAME='SST4'; end % does not exist  
if strcmp(VariableName,'qual')  CSVARNAME='Quality'; end  
if strcmp(VariableName,'sdev')  CSVARNAME='StdDev'; end  
if strcmp(VariableName,'num')   CSVARNAME='Number'; end  
if strcmp(VariableName,'flag1') CSVARNAME='Flag1'; end  
if strcmp(VariableName,'flag2') CSVARNAME='Flag2'; end  
if strcmp(VariableName,'cflag') CSVARNAME='ComFlags'; end  

% get U structure; DODS_URL,platform,year,month,day,hour
U='';
U=Get_MODIS_URLs(DATE1,DATE2,DATEINCR,Platform,TAVG,HRES,Pass,CSVARNAME);
eval(['U_' VariableName '=U;']);
        if ~isempty(U)
[kU,kU2]=size(U.day);
NU=NU+kU;
        end
    end
end

if NU==0
errordlg('Requested time interval contains no data or resolving host timed out.')
return
end

% Estimate the amount of requested data
MB=round(NU*(iLAT2-iLAT1)/LATINCR*(iLON2-iLON1)/LONINCR*16/1024/1024);
if MB>MBLIMIT
ButtonName=questdlg(['The amount of data requested is ' num2str(MB) ' MB.'], ...
        'MODIS Requested Data Amount','Continue','Cancel','Cancel');
    if strcmp(ButtonName,'Cancel') return; end
end

%COMMENTED OUT CEB 2007/05/15 hwaitbar=waitbar(0,'Acquiring ','Name','MODIS Download Progress');
kk=0;
[L1,L2]=size(Fields);
for kV=1:L1 % loop through checked variables
    if Fields{kV,2}=='y'
VariableName=Fields{kV,1};

%U structure for particular variable; DODS_URL,platform,year,month,day,hour 
eval(['U=U_' VariableName ';']);

% browse through each returned URL only if the list is not empty
        if length(U) > 0
URLs=U.DODS_URL;
[LU,L2]=size(URLs);

if SaveWorkspace=='y' & strcmp(SaveWorkspaceMode,'opendap')==1 
% find the latest opendap request number
kreq=op_getnum('opendap');
clear D
end

            for k=1:LU
URL1=deblank(URLs(k,:));

jd=datenum(U.year(k),U.month(k),U.day(k),U.hour(k),0,0);
DATE=datestr(jd,'yyyymmddHHMM');

s=[VariableName ' ' DATE];s(find(s=='_'))=' ';
kk=kk+1;
xprogr=((kk-1)/NU); % progress fraction
%COMMENTED OUT CEB 2007/05/15 waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','MODIS Download Progress');

                if strcmp(VariableName,'sst')   
clear sst

A=loaddap(['-A','+v'],URL1);
% certain files contain different data structures
    if sum(strcmp(fieldnames(A),'l3m_data')) >0
    %~isempty(strfind(URL1,'.L3m_'))
    % files like A2006094.L3m_DAY_SST_4.bz2    
    URL=[URL1 '?l3m_data' C1 C2];
    loaddap(URL)
    sst_mean=l3m_data;
    Slope=A.l3m_data.Slope;
    Intercept=A.l3m_data.Intercept;
    else
    % files like MO04MDN1.sst.ADD2000055.004.2002225153010.hdf.gz  
    URL=[URL1 '?sst_mean' C1 C2];
    loaddap(URL)
    sst_mean=Grid.Data_2520Fields.sst_mean;
    Slope=A.Global_Attributes.sst_mean.Slope;
    Intercept=A.Global_Attributes.sst_mean.Intercept;
    end 
    sst.sst=sst_mean*Slope+Intercept;
% fill missing values with NaN
    j=find(sst.sst==-300);
    sst.sst(j)=NaN;
    sst.lat=Latitude(iLAT1+1:LATINCR:iLAT2+1);
    sst.lon=Longitude(iLON1+1:LONINCR:iLON2+1);
 
sst.time=DATE;
% include attributes into the MODIS structure
sst.Attributes=A;
% add url field to the MODIS structure
sst.url=URL;

if SaveFiles=='y'
fn2=[DIRNAME '/' FNPREFIX DATE TAVGBYTE '-sst.mat']
disp(['    saving data to file: ' fn2])
save(fn2,'sst')
end
if SaveWorkspace=='y' & strcmp(SaveWorkspaceMode,'native')==1
assignin('base',['sst_' DATE TAVGBYTE],sst) 
end
if SaveWorkspace=='y' & strcmp(SaveWorkspaceMode,'opendap')==1 
D.sst=sst.sst;
D.latitude=sst.lat;
D.longitude=sst.lon;
seconds_since_1970=86400*(datenum(sst.time,'yyyymmddHHMM')-datenum('19700101000000','yyyymmddHHMMSS'));
D.time=seconds_since_1970;
D.depth = nan;
D.url=sst.url;
D.attributes=sst.Attributes;
D.name='MODIS';
D.temporal='time_series';
D.request_id = request_id1;
% readme_tmp = ['Scale: multiplied "des_stress_Liu_V" by ',...
% 	num2str(attr.des_stress_Liu_V.scale_factor)];
% readme = [readme,'/',readme_tmp];
% readme_tmp=['Comment: .'];
% readme = [readme,'/',readme_tmp];
% readme_tmp=['Scale: .'];
% readme = [readme,'/',readme_tmp];
% readme_tmp=['Warning: .'];
% readme = [readme,'/',readme_tmp];
D.readme = op_format_readme(readme); readme = ''; %USE op_format_readme(readme) TO CONVERT.
D.request_id = request_id1;
D.request_date = request_date1;
KF=num2str(k+kreq,'%04d');
WSN=['opendap_' KF];
assignin('base',WSN,D)
end
                end % if sst

                if strcmp(VariableName,'qual')   
clear qual qual1

A=loaddap(['+v','-A'],URL1);
% certain files contain different data structures
    if sum(strcmp(fieldnames(A),'l3m_data')) >0
    %~isempty(strfind(URL1,'.L3m_'))
    % files like A2006094.L3m_DAY_SST_4.bz2    
    URL=[URL1 '?l3m_data' C1 C2];
    loaddap(URL)
%    sst_mean=l3m_data;
%    Slope=A.l3m_data.Slope;
%    Intercept=A.l3m_data.Intercept;
    else
    % files like MO04MDN1.sst.ADD2000055.004.2002225153010.hdf.gz  
    URL=[URL1 '?sst_qual_b' C1 C2];
    loaddap(URL)
    qual1=Grid.Data_2520Fields.sst_qual_b;
    end 
    qual.qual=qual1;
    qual.lat=Latitude(iLAT1+1:LATINCR:iLAT2+1);
    qual.lon=Longitude(iLON1+1:LONINCR:iLON2+1);
 
qual.time = -10; %nan; %need a number here ... DATE;
qual.depth = nan;
% include attributes into the MODIS structure
qual.Attributes=A;
% add url field to the MODIS structure
qual.url=URL;

if(0)
if SaveFiles=='y'
fn2=[DIRNAME '/' FNPREFIX DATE TAVGBYTE '-qual.mat']
disp(['    saving data to file: ' fn2])
save(fn2,'qual')
end
end

if SaveWorkspace=='y' & strcmp(SaveWorkspaceMode,'native')==1
assignin('base',['qual_' DATE TAVGBYTE],qual) 
end
if SaveWorkspace=='y' & strcmp(SaveWorkspaceMode,'opendap')==1 
D.qual=qual.qual;
D.latitude=qual.lat;
D.longitude=qual.lon;
D.time=qual.time;
D.depth=qual.depth;
D.url=qual.url;
D.attributes=qual.Attributes;
D.name='MODIS';
D.temporal='time_series';
% readme_tmp = ['Scale: multiplied "des_stress_Liu_V" by ',...
% 	num2str(attr.des_stress_Liu_V.scale_factor)];
% readme = [readme,'/',readme_tmp];
% readme_tmp=['Comment: .'];
% readme = [readme,'/',readme_tmp];
% readme_tmp=['Scale: .'];
% readme = [readme,'/',readme_tmp];
% readme_tmp=['Warning: .'];
% readme = [readme,'/',readme_tmp];
D.readme = op_format_readme(readme); readme = ''; %USE op_format_readme(readme) TO CONVERT.
D.request_id = request_id1;
D.request_date = request_date1;
KF=num2str(k+kreq,'%04d');
WSN=['opendap_' KF];
assignin('base',WSN,D)
end

                end % if qual
            end % k=1:LU           
        end % if length(U) > 0
    end % if Fields Checked
end %kV    
%COMMENTED OUT CEB 2007/05/15 close(hwaitbar)

%msgbox('Data have been acquired.') COMMENTED OUT CEB 2007/05/08
end % narg

%==========================================================================
function a=Get_MODIS_URLs(DATE1,DATE2,DATEINCR,Platform,TAVG,HRES,Pass,FIELD)
               
URL_CatBase='http://dods.jpl.nasa.gov/dods-bin/nph-dods/catalogs/modis/L3';

% constrain date
CDATE=['&date("' DATE1(1:4) '/' DATE1(5:6) '/' DATE1(7:8) '","' DATE2(1:4) '/' DATE2(5:6) '/' DATE2(7:8) '")'];   

% constrain platform
if      Platform{1,2}=='n' 
        PLATFORM=Platform{2,1};
        CPLATFORM=['&platform=' '"' PLATFORM '"'];
elseif  Platform{2,2}=='n' 
        PLATFORM=Platform{1,1};
        CPLATFORM=['&platform=' '"' PLATFORM '"'];
else        
        CPLATFORM='';
end        

% constrain day/night pass
% assuming day passes return hour=12, night passes return hour=0
if      Pass{1,2}=='n'
        CPASS='&hour=0'; % only night passes
elseif  Pass{2,2}=='n'
        CPASS='&hour=12'; % only day passes
else
        CPASS='';
end        

%URL_SST=[URL_CatBase '/daily/04km/daily-04km-SST.dat' '?' CDATE]
URL1=[URL_CatBase '/' TAVG '/' HRES '/' TAVG '-' HRES '-' FIELD '.dat'];
%URL=[URL1 '?DODS_URL,DODS_JDate(modis_L3)' CDATE CPLATFORM];
URL=[URL1 '?' CDATE CPLATFORM CPASS];

%URL_StdDev=[URL_CatBase '/daily/04km/daily-04km-StdDev.dat' '?' CPLATFORM CDATE];
%URL_Quality=[URL_CatBase '/daily/04km/daily-04km-Quality.dat' '?' CPLATFORM CDATE];
%URL_Number=[URL_CatBase '/daily/04km/daily-04km-Number.dat' '?' CPLATFORM CDATE];
%A=loaddap('-A',URL)

loaddap('+v',URL)
if exist('modis_L3') & length(modis_L3.day)>0
a=modis_L3;

% Problem: for some dates there are different production versions of files.
% Solution: pick only the latest.
% Find fields that have the same time and 
% pick the latest production.
% Aqua and Terra satellites have the same date time (hour). 
% In order to distinguish between them, add small time offset (24/1000 h) to Aqua
% satellite.
[L1,L2]=size(a.platform);
for j=1:L1
    if strcmp(a.platform(j,1:4),'aqua') 
    a.hour(j)=a.hour(j)+24/1000; 
    end
end

% find and pick only the latest version
JDA=datenum(a.year,a.month,a.day,a.hour,0,0);
j=find(diff(JDA)~=0);

a.DODS_URL=a.DODS_URL(j,:);
a.platform=a.platform(j,:);
a.hour    =a.hour(j);
a.day     =a.day(j);
a.month   =a.month(j);
a.year    =a.year(j);

%% shift back the time for the aqua
%[L1,L2]=size(a.platform);
%for j=1:L1
%    if strcmp(a.platform(j,1:4),'aqua') 
%    a.hour(j)=a.hour(j)-24/1000; 
%    end
%end


else
a='';
end
%==========================================================================