function varargout=get_seawinds(varargin)
% Gets SeaWinds on Quikscat and ADEOS-II Scatterometer data.
%
% get_seawinds
% get_seawinds(request)
%
% INPUT(S):
%
%   request -- (optional) structure input which contains
%              information necessary to obtain data. The GUI
%              merely provides a graphical method of generating
%              the "request" variable. However, this visual
%              interface may be by-passed by this method.
%
% OUTPUT(S):
%
%   h -- (optional) handle to figure
%
% Example:
%
% To open up the SeaWinds interface
%
%   >> get_seawinds
%
% To obtain SeaWinds data (without the interface)
%
%   >> get_seawinds(request)
%
% where "request" is a data structure formed by the SeaWinds interface (can
% be a previous request or one altered by the user)
%
% NOTES:
%
% *ADEOS II data may not be available.
% *Wind Stress for both QuikSCAT and ADEOS II may not be available.
% *These are both due to errors found in the data at JPL/NASA but should be
% available soon.
%
% OPeNDAP Science Team
% Copyright 2007, 2008
% $Version 2.1.1$

%==========================================================================
% Christian Buckingham
%
% REVISION HISTORY: (of entire interface)
% 2007/01/01 0.x.x created, ceb
% 2007/11/21 0.x.x converting over to new data structure format contains 
%            data of interest on first level and metadata on a level deeper
%            within the data structure. (Use op_mergingtool to merge.), ceb
% 2007/12/21 0.x.x allowed code to replace no-data values with nans, ceb
% 2008/03/01 2.1.0 released as this version number (don't know why), ceb
% 2008/03/17 2.1.1 fixed enable/disable of button, had to remove ftp call 
%            and replace with static list of filenames (.mat), fix for mac
%            or if user has problem with ftp and seawinds, ceb
%==========================================================================

if nargin>0
request=varargin{1};
end

if nargin>0
% CHECK FOR CORRECTNESS OF DATA REQUEST.
% [msg,err]=get_seawinds_interimdatacheck(request);
% if (err)
%     error(msg);
%     return
% end

% Number of output.
if nargout>0
    fmt=1;
else
    fmt=0;
end

% Define parts of request structure.
%load('PointerToMainFunction.mat','main_function');
guiname = mfilename;
request.function = guiname;
request.version = op_version(mfilename);
assignin('base','request',request);

% Get data.
[msg,err,data] = SeawindsGetData(request,fmt);

% If called by gui ... and passes through the error check (or if it
% doesn't) ...
% Enable button.
if (nargin == 2)
hObject = varargin{2};
set(hObject,'Enable','On','String','Get Data'); %,'BackgroundColor',[0 1. 0.25]);
end

if (err)
    disp(msg);
    errordlg(msg);
    return
end

% If used in functional form ...
if nargout>0
    varargout={data}; %data is output.
end
end %nargin>0

if nargin == 0
%if no inputs (i.e., no "request" variable), then use
%graphical interface.
    h = gui_seawinds;
%     tmp = mfilename;
%     tmp = strrep(lower(tmp),'get','gui');
%     h = eval(tmp);
    if nargout
        varargout{1} = h; %set figure handle as output
    end
    handles = guidata(h);
    %set(handles.pushbutton7,'Enable','On','String','Get Data','BackgroundColor',[0 1. 0.25]);

end%nargin==0

return
%==========================================================================
function [msg,err,data] = SeawindsGetData(request,fmt)
% This function gets the Seawinds data from JPL.

% Get data.
% Parse vector.
DataType = request.DataType;
qscat=DataType.value(1);
adeos=DataType.value(2);

% Get quikscat data.
if (qscat)
    [msg,err,data] = qscat_get_data(request,fmt);
end
% Get adeos data.
if (adeos)
    [msg,err,data] = adeos_get_data(request,fmt);
end %adeos
%==========================================================================
function [msg,err,data]=qscat_get_data(request,fmt)
% This function retreives Seawinds on Quikscat data.
%
% Christian Buckingham
% OPeNDAP 2006

% Progress bar
hwaitbar=waitbar(0,'Acquiring ','Name','SeaWinds Download Progress');
% xprogr=(k-1)/NT; % progress fraction
% waitbar( xprogr , hwaitbar, ['Acquiring ' s ],'Name','SeaWinds Download Progress');

disp(['Please wait while data are obtained.']);

%% Set maximum amount of memory.
MBLIMIT=128;
megabyte=1024.*1024; %2.^20;
memory_limit=MBLIMIT*megabyte;

%% Parse inputs.
DATE1 = request.DATE1;
DATE2 = request.DATE2;
DATEINC = request.DATEINC;
LAT1 = request.LAT1;
LAT2 = request.LAT2;
LATINC = request.LATINC;
LON1 = request.LON1;
LON2 = request.LON2;
LONINC = request.LONINC;
DataType = request.DataType;
Field = request.Field;
Pass = request.Pass;
Attributes = request.Attributes;
SaveWorkspace = request.SaveWorkspace;
SaveWorkspaceMode = request.SaveWorkspaceMode;
SaveFile = request.SaveFile;
SaveFileType = request.SaveFileType;
directory = request.DIRNAME;
prefx = request.FNPREFIX;

SaveFileType = strrep(SaveFileType,'.','');
SaveFileType = SaveFileType(1:3);
SaveFileType = strcat('.',SaveFileType);

% The following line is to handle 'nan's.
switch_replacewithnans = 1;
count_value_changed = 0; %this keeps the original user-choice of "counts"
if switch_replacewithnans && any(Field.value(1:7)) && Field.value(8)==0
    Field.value(8)=1;
    count_value_changed = 1;
end

% Request date.
request_date1 = strrep(datestr(now,31),' ',' '); %string

%% Scaling with attributes. Scale each one individually.
switch_scale_each = 1;

%% Native.
native = strcmpi(SaveWorkspaceMode,'native');

%% Product_type.
temporal = 'time_series'; %'climatology'
    
%% This is specific to quikscat data (1) wind vectors, (2) wind stress.
DataType1=0;
DataType2=0;
if (sum(Field.value(1:8)>0)), DataType1=1; end
if (sum(Field.value(9:16)>0)), DataType2=1; end
    
%% Convert some inputs to double.
gtime1 = [str2num(DATE1(1:4)), str2num(DATE1(5:6)), str2num(DATE1(7:8))];
gtime2 = [str2num(DATE2(1:4)), str2num(DATE2(5:6)), str2num(DATE2(7:8))];
n1=datenum(gtime1);
n2=datenum(gtime2);
LAT1=str2num(LAT1);
LAT2=str2num(LAT2);
LON1=str2num(LON1);
LON2=str2num(LON2);
LATINC=str2num(LATINC);
LONINC=str2num(LONINC);

%% Check if ascending or descending.
asc=Pass.value(1);
des=Pass.value(2);

%% Check loaddap.dll file.
if(exist('loaddap','file')==0)
    msg=['loaddap not found. Make sure it is in your MATLAB path.'];
    err=1;
    data = [];
    return
end

%% URL Indices (Latitude, Longitude).
[latitude,longitude,iLAT,iLON]=latlon_seawind(LAT1,LON1,LAT2,LON2,LATINC,LONINC);     %This function generates latitude/longitude vectors
                                                                                    %and returns indices for call using opendap.
%% Define constraints.
C1=['[' num2str(iLAT(1)) ':' num2str(LATINC) ':' num2str(iLAT(2)) ']']; % ConstraintLatitude
C2=['[' num2str(iLON(1)) ':' num2str(LONINC) ':' num2str(iLON(2)) ']']; % ConstraintLongitude
C3=[];

%% Define filenames.
xprogr=0; % progress fraction
waitbar(xprogr, hwaitbar, ['Getting filenames'],'Name','SeaWinds Download Progress');

filenames1=[];
if (DataType1)
    
    URLbase = 'http://dods.jpl.nasa.gov/opendap/ocean_wind/quikscat/L3/data';
    date1 = [str2num(DATE1(1:4)), str2num(DATE1(5:6)), str2num(DATE1(7:8))];
    date2 = [str2num(DATE2(1:4)), str2num(DATE2(5:6)), str2num(DATE2(7:8))];
    daystride = 1;
    
    % Determine list of files available with their respective dates.
    database_filename = 'gui_seawinds_filelist.mat';
    if (~isdeployed)
        pname = mfilename('fullpath');
        pname = pname(1:length(pname) - length(mfilename));
        database_filename = [pname,database_filename];
    end

    status = getseawinds_updatefilelist(database_filename);

    % Load previous list of filenames and associated times.
    load('-mat',database_filename);
    
    % Define (matlab time) days of interest.
    date1_mtime = datenum(date1);
    date2_mtime = datenum(date2);
    t2_mtime = tmat(length(tmat)); %CEB 2008/06/20
    idx = find(tmat >= date1_mtime & tmat <= date2_mtime);
    dummy = filename(idx);
    
    %======================================================================
    nfiles = length(dummy);
    if ~isempty(dummy)
        nchars = length(dummy{1});
    else
        msg=['The number of files for the selection is zero. Please try again.'];
        err=1;
        data=[];
        close(hwaitbar)
        return
    end
    % Reformat cell array into a matrix.
    filenames1 = zeros(nfiles,nchars);
    for iK = 1:length(dummy)
        filenames1(iK,:)=dummy{iK};
    end
    filenames1 = char(filenames1);
    %======================================================================    
    % BELOW IS OLD ... 2008/07/09
    
%     % Method 1 (URL) - i.e., reading a webpage that has the files listed on
%     % the webpage.
%     if (1)
%         
%         % Define (matlab time) days of interest.
%         date1_mtime = datenum(date1);
%         date2_mtime = datenum(date2);
%         database_filename = 'gui_seawinds_filelist.mat';
%         if (~isdeployed)
%             pname = mfilename('fullpath');
%             pname = pname(1:length(pname) - length(mfilename));
%             database_filename = [pname,database_filename];
%         end
%         if ~exist(database_filename,'file')
%             waitbar(xprogr, hwaitbar, ...
%                 ['Generating list of filenames -- this will take a few seconds',...
%                  ''],'Name','SeaWinds Download Progress');
%             date1tmp = [1999,07,01];
%             date2tmp = datevec(now);
%             [filename,tmat,status] = ...
%                 getfile_list(URLbase,date1tmp,date2tmp,daystride);
%             save(database_filename,'filename','tmat');
%         else
%             load('-mat',database_filename);
%         end
%         %t1_mtime = datenum(tmat(1)); %t1 3-element gregorian time vector
%         t2_mtime = tmat(length(tmat)); %CEB 2008/06/20
%         %tvec = (t1_mtime:1:t2_mtime);
%         idx = find(tmat >= date1_mtime & tmat <= date2_mtime);
%         if isempty(idx)
%             dummy = getfile_list(URLbase,date1,date2,daystride);
%         elseif (date2_mtime - t2_mtime) > 0
%             dummy = filename(idx);
%             date1 = datevec(t2_mtime + 1);
%             %date2 = date2;
%             tmp = getfile_list(URLbase,date1,date2,daystride);
%             if ~isempty(tmp)
%             for ii = length(dummy)
%                 dummy{length(dummy)+1} = tmp{ii};
%             end
%             end
%         else
%             dummy = filename(idx);
%         end
%     end
%     
%     % Method 2 (FTP).
%     if (0)
%         ftphost = 'podaac.jpl.nasa.gov';
%         toplevel = '/pub/ocean_wind/quikscat/L3/data/';
%         dummy = getfile_list(ftphost,toplevel,date1,date2,daystride);
%     end
%     
%     %======================================================================
%     nfiles = length(dummy);
%     if ~isempty(dummy)
%         nchars = length(dummy{1});
%     else
%         msg=['The number of files for the selection is zero. Please try again.'];
%         err=1;
%         data=[];
%         close(hwaitbar)
%         return
%     end
%     % Reformat cell array into a matrix.
%     filenames1 = zeros(nfiles,nchars);
%     for iK = 1:length(dummy)
%         filenames1(iK,:)=dummy{iK};
%     end
%     filenames1 = char(filenames1);
%     %======================================================================
%     
%     % Method 3 (CATALOG).
%     if (0)
%         filenames1=qscat_get_filenames_wind(DATE1,DATE2,DATEINC); %This function determines filenames.
%     end
end

% The following is not applicable right now because the wind stress vectors
% are not available 2008/03/01.
filenames2=[];
if (DataType2)
    filenames2=qscat_get_filenames_stress(DATE1,DATE2,DATEINC); %This function determines filenames.
end

% Handle errors.--> cannot connect to catalog.
err1=0;
if (DataType1 && isempty(filenames1))
    err1=1;
end
err2=0;
if (DataType2 && isempty(filenames2))
    err2=1;
end
if (DataType1 && ~DataType2 && err1)
    msg=['The number of files for "Wind Vectors" is zero or',...
        ' a connection could not be made. Please try again at a future date.'];
    err=1;
    data=[];
    close(hwaitbar); return
end
if (DataType2 && ~DataType1 && err2)
    msg=['The number of files for "Wind Stress" is zero or',...
        ' a connection could not be made. Please try again at a future date.'];
    err=1;
    data=[];
    close(hwaitbar); return
end
if (DataType1 && DataType2 && err1 && err2)
    msg=['The number of files for "Wind Vectors" and "Wind Stress" is zero or',...
        ' a connection could not be made. Please try again at a future date.'];
    err=1;
    data=[];
    close(hwaitbar); return
end

% Estimate amount of data.
nfile=size(filenames1,1);
npass=asc+des; %1 if just asc or des, 2 if both. Number of passes.
nvariables = sum(Field.value(1:8)); %number of variables.
nlat=length(latitude);
nlon=length(longitude);
nbyt1=8*nlat*nlon*nfile*npass*nvariables;

% Estimate amount of data.
nfile=size(filenames2,1);
npass=asc+des; %1 if just asc or des, 2 if both. Number of passes.
nvariables = sum(Field.value(9:16)); %number of variables.
nlat=length(latitude);
nlon=length(longitude);
nbyt2=8*nlat*nlon*nfile*npass*nvariables;

if SaveWorkspace
if (nbyt1 + nbyt2 > memory_limit)
    msg=['The amount of required memory exceeds the specified memory limit of ',...
        num2str(MBLIMIT),' MB. What would you like to do?'];
    ButtonName=questdlg(msg, ...
                           'Memory Limit Exceeded', ...
                           'Continue','Stop','Continue');
    switch ButtonName,
        case 'Continue',
            
        case 'Stop',
            msg=['You have chosen to stop execution of the program.'];
            err=1;
            data=[];
            close(hwaitbar); return
    end % switch
    
    pause(.1);
    
end
end

%% Ascending and Descending time.
time_frc_asc = 0.25; %06/24
time_frc_des = 0.75; %(6 + 12)/24

%% Get opendap structure number.
count= op_getnum('opendap','base');
count = count+1;

%% Attributes.
nfile1=size(filenames1,1);
nfile2=size(filenames2,1);
if (~switch_scale_each)
    if (DataType1 && nfile1>0)
    url = filenames1(1,:); %use first file as template
    attr1 = loaddap('-A',url); % get attributes
    end
    if (DataType2 && nfile2>0)
    url = filenames2(1,:); %use first file as template
    attr2 = loaddap('-A',url); % get attributes
    end
end%attributes

%% Initialize "readme."
readme = '';

%% Define number of loaddap calls.
npass = length(Pass.value);
npass_true = Pass.value(1)+Pass.value(2); %both asc and des
ncall = npass_true*(nfile1+nfile2);

%% DataType1 --> Wind Vectors.
nfile1=size(filenames1,1);
if (DataType1 && nfile1>0)
    % Loop.
    name='qscat';
    for iK=1:nfile1
        for iL=1:npass %ascending,descending
        
        if (~Pass.value(iL))
            continue %skip the following code.
        else %if statement over entire read/write code.
            
        url=filenames1(iK,:);
        CE=[];
        if (~switch_scale_each)
            attr = attr1;
        else%Attributes
            attr=loaddap('-A',url); % get attributes
        end%Attributes
        
        % Load data using url protocol.
        if (iL == 1) %ascending
            
            iloc=strfind(url,'QS_XWGRD3');
            s = url(iloc:end);
            s = strrep(s,'_',' ');
            xprogr=(iK+iL/npass_true - 1.5)/nfile1; % progress fraction
            waitbar(xprogr , hwaitbar, ['Acquiring ' s ,' ascending'],'Name','SeaWinds Download Progress');
            
            time_frc = time_frc_asc;
            
            if (Field.value(1))
                CE=[CE,'asc_avg_wind_speed',C1,C2,','];
            end
            if (Field.value(2))
                CE=[CE,'asc_avg_wind_vel_u',C1,C2,','];
            end
            if (Field.value(3))
                CE=[CE,'asc_avg_wind_vel_v',C1,C2,','];
            end
            if (Field.value(4))
                CE=[CE,'asc_avg_wind_speed_sq',C1,C2,','];
            end
            if (Field.value(5))
                CE=[CE,'asc_time_frac',C1,C2,','];
            end
            if (Field.value(6))
                CE=[CE,'asc_rain_flag',C1,C2,','];
            end
            if (Field.value(7))
                CE=[CE,'asc_rain_prob',C1,C2,','];
            end
            if (Field.value(8))
                CE=[CE,'asc_wvc_count',C1,C2,','];
            end
            len=length(CE);
            CE=CE(1:len-1); %remove comma.

            % Load data.
            disp([url,'?',CE])
            if (~native)
                loaddap('-e',[url,'?',CE]);
            else%native
                data_tmp = loaddap([url,'?',CE]);
            end%native

            % Scale data.
            if (~native)
            
            if switch_replacewithnans
                nan_count = nan(size(asc_wvc_count));
                nan_count(asc_wvc_count==1) = 1;
            else
                nan_count = ones(size(asc_wvc_count));
            end
                
            K=1;
            if (exist('asc_avg_wind_speed','var')),
                avg_wind_speed_mat(K,:,:)= ...
                    asc_avg_wind_speed.*...
                    attr.asc_avg_wind_speed.scale_factor.*nan_count;
            end
            if (exist('asc_avg_wind_speed_sq','var')),
                avg_wind_speed_sq_mat(K,:,:)= ...
                    asc_avg_wind_speed_sq.*...
                    attr.asc_avg_wind_speed_sq.scale_factor.*nan_count;
            end
            if (exist('asc_avg_wind_vel_u','var')),
                avg_wind_vel_u_mat(K,:,:)= ...
                    asc_avg_wind_vel_u.*...
                    attr.asc_avg_wind_vel_u.scale_factor.*nan_count;
            end
            if (exist('asc_avg_wind_vel_v','var')),
                avg_wind_vel_v_mat(K,:,:)= ...
                    asc_avg_wind_vel_v.*...
                    attr.asc_avg_wind_vel_v.scale_factor.*nan_count;
            end
            if (exist('asc_rain_flag','var')),
                rain_flag_mat(K,:,:)= ...
                    asc_rain_flag.*...
                    attr.asc_rain_flag.scale_factor.*nan_count;
            end
            if (exist('asc_rain_prob','var')),
                rain_prob_mat(K,:,:)= ...
                    asc_rain_prob.*...
                    attr.asc_rain_prob.scale_factor.*nan_count;
            end
            if (exist('asc_time_frac','var')),
                time_frac_mat(K,:,:)= ... 
                    asc_time_frac.*...
                    attr.asc_time_frac.scale_factor.*nan_count;
            end
            if (exist('asc_wvc_count','var')),
                wvc_count_mat(K,:,:)= ...
                    asc_wvc_count.*...
                    attr.asc_wvc_count.scale_factor.*nan_count;
            end
            
            end%native
                        
        end %asc
        
        if (iL == 2) %descending
            
            iloc=strfind(url,'QS_XWGRD3');
            s = url(iloc:end);
            s = strrep(s,'_',' ');
            xprogr=(iK+iL/npass_true - 1.5)/nfile1; % progress fraction
            waitbar(xprogr , hwaitbar, ['Acquiring ' s ,' descending'],'Name','SeaWinds Download Progress');
            
            time_frc = time_frc_des;
            
            if (Field.value(1))
                CE=[CE,'des_avg_wind_speed',C1,C2,','];
            end
            if (Field.value(2))
                CE=[CE,'des_avg_wind_vel_u',C1,C2,','];
            end
            if (Field.value(3))
                CE=[CE,'des_avg_wind_vel_v',C1,C2,','];
            end
            if (Field.value(4))
                CE=[CE,'des_avg_wind_speed_sq',C1,C2,','];
            end
            if (Field.value(5))
                CE=[CE,'des_time_frac',C1,C2,','];
            end
            if (Field.value(6))
                CE=[CE,'des_rain_flag',C1,C2,','];
            end
            if (Field.value(7))
                CE=[CE,'des_rain_prob',C1,C2,','];
            end
            if (Field.value(8))
                CE=[CE,'des_wvc_count',C1,C2,','];
            end
            len=length(CE);
            CE=CE(1:len-1); %remove comma.
            
            % Load data.
            disp([url,'?',CE])
            if (~native)
                loaddap([url,'?',CE]);
            else%native
                data_tmp = loaddap([url,'?',CE]);
            end%native

            % Scale data.
            if (~native)
            
            if switch_replacewithnans
                nan_count = nan(size(des_wvc_count));
                nan_count(des_wvc_count==1) = 1;
            else
                nan_count = ones(size(des_wvc_count));
            end
            
            K=1;
            if (exist('des_avg_wind_speed','var')),
                avg_wind_speed_mat(K,:,:)= ...
                    des_avg_wind_speed.*...
                    attr.des_avg_wind_speed.scale_factor.*nan_count;
            end
            if (exist('des_avg_wind_speed_sq','var')),
                avg_wind_speed_sq_mat(K,:,:)= ...
                    des_avg_wind_speed_sq.*...
                    attr.des_avg_wind_speed_sq.scale_factor.*nan_count;
            end
            if (exist('des_avg_wind_vel_u','var')),
                avg_wind_vel_u_mat(K,:,:)= ...
                    des_avg_wind_vel_u.*...
                    attr.des_avg_wind_vel_u.scale_factor.*nan_count;
            end
            if (exist('des_avg_wind_vel_v','var')),
                avg_wind_vel_v_mat(K,:,:)= ...
                    des_avg_wind_vel_v.*...
                    attr.des_avg_wind_vel_v.scale_factor.*nan_count;
            end
            if (exist('des_rain_flag','var')),
                rain_flag_mat(K,:,:)= ...
                    des_rain_flag.*...
                    attr.des_rain_flag.scale_factor.*nan_count;
            end
            if (exist('des_rain_prob','var')),
                rain_prob_mat(K,:,:)= ...
                    des_rain_prob.*...
                    attr.des_rain_prob.scale_factor.*nan_count;
            end
            if (exist('des_time_frac','var')),
                time_frac_mat(K,:,:)= ... 
                    des_time_frac.*...
                    attr.des_time_frac.scale_factor.*nan_count;
            end
            if (exist('des_wvc_count','var')),
                wvc_count_mat(K,:,:)= ...
                    des_wvc_count.*...
                    attr.des_wvc_count.scale_factor.*nan_count;
            end
            
            end%native
        end %des
        
        % Save data in structure.
        if (~native)
        if (exist('avg_wind_speed_mat','var')),     data_tmp.wind_sp=squeeze(avg_wind_speed_mat); end
        if (exist('avg_wind_speed_sq_mat','var')),  data_tmp.wind_sp_sq=squeeze(avg_wind_speed_sq_mat); end
        if (exist('avg_wind_vel_u_mat','var')),     data_tmp.wind_u=squeeze(avg_wind_vel_u_mat); end
        if (exist('avg_wind_vel_v_mat','var')),     data_tmp.wind_v=squeeze(avg_wind_vel_v_mat); end
        if (exist('rain_flag_mat','var')),          data_tmp.rain_flag=squeeze(rain_flag_mat); end
        if (exist('rain_prob_mat','var')),          data_tmp.rain_prob=squeeze(rain_prob_mat); end
        if (exist('time_frac_mat','var')),          data_tmp.time_frac=squeeze(time_frac_mat); end
        if count_value_changed==0
            if (exist('wvc_count_mat','var')),          data_tmp.count=squeeze(wvc_count_mat); end
        end
        end%native
        
        % time, date.
        iloc=strfind(url,'QS_XWGRD3');
        yyyyddd=url(iloc+10:iloc+16);
        %greg2 starts at yearday = 0.0.
        gtime = op_greg2(str2double(yyyyddd(5:7))-1 + time_frc,str2double(yyyyddd(1:4)));
        time = op_epoch(gtime);
        readme_tmp = ['time in seconds since reference time; '];
                readme = [readme,readme_tmp];
        readme_tmp = ['time in sun-synchronous time; '];
                readme = [readme,readme_tmp];
        readme_tmp = ['data have been scaled; '];
                readme = [readme,readme_tmp];
        %readme_tmp = ['missing values replaced with NaN; '];
        %        readme = [readme,readme_tmp];
        if strcmp(readme(end-1:end),'; ')
            readme = readme(1:end-2);
        end
        variable_name=['opendap','_',num2str(count,'%04d')];
        yyyymmdd=[num2str(gtime(1),'%04d'),...
                  num2str(gtime(2),'%02d'),...
                  num2str(gtime(3),'%02d')];
        if (native)
            if (iL==1)
                pass_txt = 'a';
            else
                pass_txt = 'd';
            end
            variable_name=[name,'_',yyyymmdd,pass_txt];
        end
        count = count+1;
        
        if (exist('data_tmp','var'))
            err=0;
            msg='';
        else
            data=[];
            err=1;
            msg=['Either no selections were made as to the variables OR',...
                ' there is something wrong with "loaddap". "Wind Vector" data have not been obtained.'];
            return
        end

        % Dependent variables.
        % Independent variables
        % Metadata.
        if (~native)
        
        % Dependent variables..
        depend_fields = fieldnames(data_tmp);
        
        % Independent variables.
        data_tmp.latitude=latitude;
        data_tmp.longitude=longitude;
        data_tmp.time=time;
        %data_tmp.depth=nan;
        
        % Metadata.
        tt = sprintf('%s',''''); %SINGLE QUOTE MARK
        str = ['{',tt,'latitude',tt,...
              ',',tt,'longitude',tt,...
              '}'];
        for idep=1:length(depend_fields)
            tmptmp = depend_fields{idep};
            eval(['data_tmp.metadata','.',tmptmp,'_variables',...
                 '=',str,';']);
        end
        data_tmp.metadata.url=[url,'?',CE]; %data_tmp.ce=['?',CE];
        if (Attributes)
            data_tmp.metadata.attributes=attr;
        end%attributes
        data_tmp.metadata.name=name;
        data_tmp.metadata.temporal = temporal;
        data_tmp.metadata.reference_time = datestr([1970,01,01,00,00,00],31);
        data_tmp.metadata.readme = readme; readme = '';
        data_tmp.metadata.user_friendly_time = datestr(gtime,31);
        %data_tmp.request_id = request_id1;
        data_tmp.metadata.request_date = request_date1;
        data_tmp.metadata.request = request;
        end
        
        % Put data in base workspace.
        if (SaveWorkspace && fmt==0)
            ws='base';
            assignin(ws,variable_name, data_tmp);
            %disp(['Writing variable "',variable_name,'" to current workspace.']);
        end %SaveWorkspace

        % Write ".mat" file.
        if (SaveFile && fmt==0 && strcmpi(SaveFileType,'.mat'))
            eval([variable_name,'=','data_tmp;']);
            slash='/';
            %if (isempty(prefx)), prefx=[name,'_']; end
            if (isempty(directory)), directory=pwd; end
            if (ispc), slash='\'; end
            if (isunix), slash='/'; end
            %if (strcmp(computer,'mac') || strcmp(computer,'maci')), slash='/'; end
            if (native)
                fname=[prefx,yyyymmdd,'.mat'];
            else
                fname=[prefx,num2str(count-1,'%04d'),'.mat'];
            end
            disp(['Writing to file ... ',fname]);
            waitbar(xprogr, hwaitbar,...
                ['Writing to file ... ',strrep(fname,'_',' ')],...
                'Name','SeaWinds Download Progress');
            if native && exist([directory,slash,fname],'file')
                % This is for use with ascending and descending passes
                % in "native" mode -- both passes in same .mat file.
                save([directory,slash,fname],variable_name,'-append');
            else
                save([directory,slash,fname],variable_name);
            end
            eval(['clear ',variable_name]);
        end %.mat
        
        % Write ".txt" file.
        % This section writes to an arcview-compatible grid file.
        % It requires an external function called arcgridwrite.
        if (SaveFile && fmt==0 && strcmpi(SaveFileType,'.txt'))
            %eval([variable_name,'=','data_tmp;']);
            slash='/';
            %if (isempty(prefx)), prefx=[name,'_']; end
            if (isempty(directory)), directory=pwd; end
            if (ispc), slash='\'; end
            if (isunix), slash='/'; end
            %if (strcmp(computer,'mac') || strcmp(computer,'maci')), slash='/'; end
            %subdir='';
            if ~exist([directory],'dir')
                mkdir([directory])
            end
            if (iL==1)
                pass_txt = 'a';
            else
                pass_txt = 'd';
            end
            x = data_tmp.longitude;
            y = data_tmp.latitude;
            [X,Y] = meshgrid(x,y);
            field_names = fieldnames(data_tmp);
            
            % Define "token fields".
            ii=1;
            token_fields{ii}='latitude'; ii=ii+1;
            token_fields{ii}='longitude'; ii=ii+1;
            token_fields{ii}='time'; ii=ii+1;
            token_fields{ii}='depth'; ii=ii+1;
            token_fields{ii}='name'; ii=ii+1;
            token_fields{ii}='url'; ii=ii+1;
            token_fields{ii}='attributes'; ii=ii+1;
            token_fields{ii}='temporal'; ii=ii+1;
            token_fields{ii}='readme'; ii=ii+1;
            token_fields{ii}='request_id'; ii=ii+1;
            token_fields{ii}='metadata'; ii=ii+1;
            token_fields{ii}='request_date'; ii=ii+1;
            
            % Loop over fieldnames.
            for ifield = 1:length(field_names) % ifield -- index fieldname
            
            tmpf = field_names{ifield};
            % If is not a token field ...
            if ~any(ismember(token_fields,tmpf))
                           
                eval(['Z = data_tmp.',tmpf,';'])
                fname=[directory,slash,...
                       prefx,yyyymmdd,'_',pass_txt,'_',tmpf,'.txt'];
                disp(['Writing variables to file ... ',fname]);
                arcgridwrite(fname,X,Y,Z);
                
            end
            
            end % ifield -- index fieldname
            eval(['clear ',variable_name]);
            
        end %.txt
        
%         % Pass data up to user.
%         if (fmt==1)
%             eval(['data.',variable_name,'=','data_tmp;']);
%             disp(['Placing variables in larger structure ... ',variable_name]);
%             eval(['clear ',variable_name]);
%         end %SaveWorkspace

        end%if statement over entire read/write code.
        
        % Remove data structure.
        if (exist('avg_wind_speed_mat','var')),     clear avg_wind_speed_mat; end
        if (exist('avg_wind_speed_sq_mat','var')),  clear avg_wind_speed_sq_mat; end
        if (exist('avg_wind_vel_u_mat','var')),     clear avg_wind_vel_u_mat; end
        if (exist('avg_wind_vel_v_mat','var')),     clear avg_wind_vel_v_mat; end
        if (exist('rain_flag_mat','var')),          clear rain_flag_mat; end
        if (exist('rain_prob_mat','var')),          clear rain_prob_mat; end
        if (exist('time_frac_mat','var')),          clear time_frac_mat; end
        if (exist('wvc_count_mat','var')),          clear wvc_count_mat; end
        
        if (exist('data_tmp','var')), clear data_tmp; end
        
        end %loop over orbit
    end %loop over files
end %DataType1.

% Remove data structure.
if (exist('avg_wind_speed_mat','var')),     clear avg_wind_speed_mat; end
if (exist('avg_wind_speed_sq_mat','var')),  clear avg_wind_speed_sq_mat; end
if (exist('avg_wind_vel_u_mat','var')),     clear avg_wind_vel_u_mat; end
if (exist('avg_wind_vel_v_mat','var')),     clear avg_wind_vel_v_mat; end
if (exist('rain_flag_mat','var')),          clear rain_flag_mat; end
if (exist('rain_prob_mat','var')),          clear rain_prob_mat; end
if (exist('time_frac_mat','var')),          clear time_frac_mat; end
if (exist('wvc_count_mat','var')),          clear wvc_count_mat; end

if (exist('data_tmp','var')), clear data_tmp; end

%% DataType2 --> Wind Stress.
% THIS SECTION LIKELY DOES NOT WORK BECAUSE HAD TO UPDATE WIND VECTOR
% SECTION WHILE STRESS AND ADEOS (WIND VECTOR, STRESS) WERE DOWN. SO THE
% CODE PERTAINING TO QSCAT STRESS AND ADEOS VECTOR/STRESS ARE LIKELY OUT OF
% DATE. 2008/07/09.
nfile2=size(filenames2,1);
if (DataType2 && nfile2>0)
    % Loop.
    name='qscat_stress';
    for iK=1:nfile2
        npass = length(Pass.value);
        for iL=1:npass %ascending,descending
            
        if (~Pass.value(iL))
            continue %skip the following code.
        else %if statement over entire read/write code.
                    
        url=filenames2(iK,:);
        CE=[];
        if (~switch_scale_each)
            attr = attr2;
        else%Attributes
            attr=loaddap('-A',url); % get attributes
        end%Attributes
        
        if (iL == 1) %ascending
            
            time_frc = time_frc_asc;

            % Load data using url protocol.
            if (Field.value(9))
                CE=[CE,'asc_stress_Large_U',C1,C2,','];
            end
            if (Field.value(10))
                CE=[CE,'asc_stress_Large_V',C1,C2,','];
            end
            if (Field.value(11))
                CE=[CE,'asc_stress_Liu_U',C1,C2,','];
            end
            if (Field.value(12))
                CE=[CE,'asc_stress_Liu_V',C1,C2,','];
            end
            if (Field.value(13))
                CE=[CE,'asc_cd_Large',C1,C2,','];
            end
            if (Field.value(14))
                CE=[CE,'asc_cd_Liu',C1,C2,','];
            end
            if (Field.value(6))
                CE=[CE,'asc_rain_flag',C1,C2,','];
            end
            if (Field.value(5))
                CE=[CE,'asc_time_frac',C1,C2,','];
            end
            len=length(CE);
            CE=CE(1:len-1); %remove comma.

            % Load data.
            disp([url,'?',CE])
            if (~native)
                loaddap([url,'?',CE]);
            else%native
                data_tmp = loaddap([url,'?',CE]);
            end%native

            % Scale data.
            if (~native)
            K=1;
            if (exist('asc_cd_Large','var')),
                cd_Large_mat(K,:,:)= ...
                    asc_cd_Large.*attr.asc_cd_Large.scale_factor;
            end
            if (exist('asc_cd_Liu','var')),
                cd_Liu_mat(K,:,:)= ...
                    asc_cd_Liu.*attr.asc_cd_Liu.scale_factor;
            end
            if (exist('asc_rain_flag','var')),
                rain_flag_mat(K,:,:)= ...
                    asc_rain_flag.*attr.asc_rain_flag.scale_factor;
            end
            if (exist('asc_stress_Large_U','var')),
                stress_Large_U_mat(K,:,:)= ...
                    asc_stress_Large_U.*attr.asc_stress_Large_U.scale_factor;
            end
            if (exist('asc_stress_Large_V','var')),
                stress_Large_V_mat(K,:,:)= ...
                    asc_stress_Large_V.*attr.asc_stress_Large_V.scale_factor;
            end
            if (exist('asc_stress_Liu_U','var')),
                stress_Liu_U_mat(K,:,:)= ...
                    asc_stress_Liu_U.*attr.asc_stress_Liu_U.scale_factor;
            end
            if (exist('asc_stress_Liu_V','var')),
                stress_Liu_V_mat(K,:,:)= ...
                    asc_stress_Liu_V.*attr.asc_stress_Liu_V.scale_factor;
            end
            if (exist('asc_time_frac','var')),
                time_frac_mat(K,:,:)= ...
                    asc_time_frac.*attr.asc_time_frac.scale_factor;
            end
            
            end%native
        
        end%asc
        
        if (iL == 2) %descending
            
            time_frc = time_frc_des;
            
            % Load data using url protocol.
            if (Field.value(9))
                CE=[CE,'des_stress_Large_U',C1,C2,','];
            end
            if (Field.value(10))
                CE=[CE,'des_stress_Large_V',C1,C2,','];
            end
            if (Field.value(11))
                CE=[CE,'des_stress_Liu_U',C1,C2,','];
            end
            if (Field.value(12))
                CE=[CE,'des_stress_Liu_V',C1,C2,','];
            end
            if (Field.value(13))
                CE=[CE,'des_cd_Large',C1,C2,','];
            end
            if (Field.value(14))
                CE=[CE,'des_cd_Liu',C1,C2,','];
            end
            if (Field.value(6))
                CE=[CE,'des_rain_flag',C1,C2,','];
            end
            if (Field.value(5))
                CE=[CE,'des_time_frac',C1,C2,','];
            end
            len=length(CE);
            CE=CE(1:len-1); %remove comma.
            
            % Load data.
            disp([url,'?',CE])
            if (~native)
                loaddap([url,'?',CE]);
            else%native
                data_tmp = loaddap([url,'?',CE]);
            end%native

            % Scale data.
            if (~native)
            K=1;
            if (exist('des_cd_Large','var')),
                cd_Large_mat(K,:,:)= ...
                    des_cd_Large.*attr.des_cd_Large.scale_factor;
            end
            if (exist('des_cd_Liu','var')),
                cd_Liu_mat(K,:,:)= ...
                    des_cd_Liu.*attr.des_cd_Liu.scale_factor;
            end
            if (exist('des_rain_flag','var')),
                rain_flag_mat(K,:,:)= ...
                    des_rain_flag.*attr.des_rain_flag.scale_factor;
            end
            if (exist('des_stress_Large_U','var')),
                stress_Large_U_mat(K,:,:)= ...
                    des_stress_Large_U.*attr.des_stress_Large_U.scale_factor;
            end
            if (exist('des_stress_Large_V','var')),
                stress_Large_V_mat(K,:,:)= ...
                    des_stress_Large_V.*attr.des_stress_Large_V.scale_factor;
            end
            if (exist('des_stress_Liu_U','var')),
                stress_Liu_U_mat(K,:,:)= ...
                    des_stress_Liu_U.*attr.des_stress_Liu_U.scale_factor;
            end
            if (exist('des_stress_Liu_V','var')),
                stress_Liu_V_mat(K,:,:)= ...
                    des_stress_Liu_V.*attr.des_stress_Liu_V.scale_factor;
            end
            if (exist('des_time_frac','var')),
                time_frac_mat(K,:,:)= ...
                    des_time_frac.*attr.des_time_frac.scale_factor;
            end
            
            end%native
        
        end %des

        % Save data in structure..
        if (~native)
        if (exist('stress_Large_U_mat','var')),     data_tmp.stress_large_u=squeeze(stress_Large_U_mat); end
        if (exist('stress_Large_V_mat','var')),     data_tmp.stress_large_v=squeeze(stress_Large_V_mat); end
        if (exist('stress_Liu_U_mat','var')),       data_tmp.stress_liu_u=squeeze(stress_Liu_U_mat); end
        if (exist('stress_Liu_V_mat','var')),       data_tmp.stress_liu_v=squeeze(stress_Liu_V_mat); end
        if (exist('cd_Large_mat','var')),           data_tmp.cd_large=squeeze(cd_Large_mat); end
        if (exist('cd_Liu_mat','var')),             data_tmp.cd_liu=squeeze(cd_Liu_mat); end
        if (exist('rain_flag_mat','var')),          data_tmp.rain_flag=squeeze(rain_flag_mat); end
        if (exist('time_frac_mat','var')),          data_tmp.time_frac=squeeze(time_frac_mat); end
        end%native
        
        % time, date.
        iloc=strfind(url,'QS_STGRD3');
        yyyyddd=url(iloc+10:iloc+16);
        gtime = op_greg2(str2double(yyyyddd(5:7))-1 + time_frc,str2double(yyyyddd(1:4)));
        %op_greg2 starts at yearday = 0.0.
        time = op_epoch(gtime);
        readme_tmp = ['time in seconds since reference time; '];
                readme = [readme,readme_tmp];
        readme_tmp = ['time in sun-synchronous time; '];
                readme = [readme,readme_tmp];
        %readme_tmp = ['missing values replaced with NaN; '];
        %        readme = [readme,readme_tmp];
        variable_name=['opendap','_',num2str(count,'%04d')];
        if (native)
            if (iL==1)
                pass_txt = 'a';
            else
                pass_txt = 'd';
            end
            variable_name=[name,'_',yyyymmdd,pass_txt];
        end
        count = count+1;
        
        if (exist('data_tmp','var'))
            err=0;
            msg='';
        else
            data=[];
            err=1;
            msg=['Either no selections were made as to the variables OR',...
                ' there is something wrong with "loaddap". "Wind Stress" data have not been obtained.'];
            return
        end

        % Dependent variables.
        % Independent variables
        % Metadata.
        if (~native)
        
        % Dependent variables..
        depend_fields = fieldnames(data_tmp);
        
        % Independent variables.
        data_tmp.latitude=latitude;
        data_tmp.longitude=longitude;
        data_tmp.time=time;
        %data_tmp.depth=nan;
        
        % Metadata.
        tt = sprintf('%s',''''); %SINGLE QUOTE MARK
        str = ['{',tt,'latitude',tt,...
              ',',tt,'longitude',tt,...
              '}'];
        for idep=1:length(depend_fields)
            tmptmp = depend_fields{idep};
            eval(['data_tmp.metadata','.',tmptmp,'_variables',...
                 '=',str,';']);
        end
        data_tmp.metadata.url=[url,'?',CE]; %data_tmp.ce=['?',CE];
        if (Attributes)
            data_tmp.metadata.attributes=attr;
        end%attributes
        data_tmp.metadata.name=name;
        data_tmp.metadata.temporal = temporal;
        data_tmp.metadata.reference_time = datestr([1970,01,01,00,00,00],31);
        data_tmp.metadata.readme = readme; readme = '';
        data_tmp.metadata.user_friendly_time = datestr(gtime,31);
        %data_tmp.request_id = request_id1;
        data_tmp.metadata.request_date = request_date1;
        data_tmp.metadata.request = request;
        end

        % Put data in base workspace.
        if (SaveWorkspace && fmt==0)
            ws='base';
            assignin(ws,variable_name,data_tmp);
            %disp(['Writing variable "',variable_name,'" to current workspace.']);
        end %SaveWorkspace

        % Write ".mat" file.
        if (SaveFile && fmt==0 && strcmpi(SaveFileType,'.mat'))
            eval([variable_name,'=','data_tmp;']);
            slash='/';
            %if (isempty(prefx)), prefx=[name,'_']; end
            if (isempty(directory)), directory=pwd; end
            if (ispc), slash='\'; end
            if (isunix), slash='/'; end
            %if (strcmp(computer,'mac') || strcmp(computer,'maci')), slash='/'; end
            fname=[directory,slash,prefx,yyyymmdd,'.mat'];
            save(fname,variable_name);
            disp(['Writing variables to file ... ',fname]);
            eval(['clear ',variable_name]);
        end %.mat
        
%         % Pass data up to user.
%         if (fmt==1)
%             eval(['data.',variable_name,'=','data_tmp;']);
%             disp(['Placing variables in larger structure ... ',variable_name]);
%             eval(['clear ',variable_name]);
%         end %SaveWorkspace

        end%if statement over entire read/write code.
        
        % Remove data structure.
        if (exist('avg_wind_speed_mat','var')),     clear avg_wind_speed_mat; end
        if (exist('avg_wind_speed_sq_mat','var')),  clear avg_wind_speed_sq_mat; end
        if (exist('avg_wind_vel_u_mat','var')),     clear avg_wind_vel_u_mat; end
        if (exist('avg_wind_vel_v_mat','var')),     clear avg_wind_vel_v_mat; end
        if (exist('rain_flag_mat','var')),          clear rain_flag_mat; end
        if (exist('rain_prob_mat','var')),          clear rain_prob_mat; end
        if (exist('time_frac_mat','var')),          clear time_frac_mat; end
        if (exist('wvc_count_mat','var')),          clear wvc_count_mat; end
        
        if (exist('data_tmp','var')), clear data_tmp; end

        end %loop over orbit
    end %loop over files
end %DataType2.

msg='';
err=0;
if ((nfile1==0 && nfile2==0) || fmt==0)
    data=[];
end
%data=[];    %change this if one wants to pass up the data through the functions.
            %If you want to do this, place DataType1 and DataType2 in a
            %single variable called "data", otherwise only wind stress will
            %be brought up to the top.

% if request.SaveWorkspace
%     disp(['The data have been acquired.']);
% elseif request.SaveFile
%     disp(['The data have been acquired.'])
%     %disp([directory]);
% else
%     disp(['The data have been acquired.'])
% end

close(hwaitbar)
disp(['The data have been acquired.'])

return %endoffunction
%==========================================================================
function [msg,err,data]=adeos_get_data(request,fmt)
% This function retreives Seawinds on ADEOS II data.
%
% Christian Buckingham
% OPeNDAP 2006

disp(['Please wait while data are obtained.']);

%% Set maximum amount of memory.
MBLIMIT=128;
megabyte=1024.*1024; %2.^20;
memory_limit=MBLIMIT*megabyte;

%% Parse inputs.
DATE1 = request.DATE1;
DATE2 = request.DATE2;
DATEINC = request.DATEINC;
LAT1 = request.LAT1;
LAT2 = request.LAT2;
LATINC = request.LATINC;
LON1 = request.LON1;
LON2 = request.LON2;
LONINC = request.LONINC;
DataType = request.DataType;
Field = request.Field;
Pass = request.Pass;
Attributes = request.Attributes;
SaveWorkspace = request.SaveWorkspace;
SaveWorkspaceMode = request.SaveWorkspaceMode;
SaveFile = request.SaveFile;
SaveFileType = request.SaveFileType;
directory = request.DIRNAME;
prefx = request.FNPREFIX;

SaveFileType = strrep(SaveFileType,'.','');
SaveFileType = SaveFileType(1:3);
SaveFileType = strcat('.',SaveFileType);

% Request date.
request_date1 = strrep(datestr(now,31),' ',' '); %string

%% Scaling with attributes. Scale each one individually.
switch_scale_each = 1;

%% Native.
native = strcmp(lower(SaveWorkspaceMode),'native');

%% Product_type.
temporal = 'time_series'; %'climatology'
    
%% This is specific to adeos data (1) wind vectors, (2) wind stress.
DataType1=0;
DataType2=0;
if (sum(Field.value(1:11)>0)), DataType1=1; end
if (sum(Field.value(12:19)>0)), DataType2=1; end
    
%% Convert some inputs to double.
n1=datenum([str2num(DATE1(1:4)), str2num(DATE1(5:6)), str2num(DATE1(7:8))]);
n2=datenum([str2num(DATE2(1:4)), str2num(DATE2(5:6)), str2num(DATE2(7:8))]);
LAT1=str2num(LAT1);
LAT2=str2num(LAT2);
LON1=str2num(LON1);
LON2=str2num(LON2);
LATINC=str2num(LATINC);
LONINC=str2num(LONINC);

%% Check if ascending or descending.
asc=Pass.value(1);
des=Pass.value(2);

%% Check loaddap.dll file.
if(exist('loaddap','file')==0)
    msg=['loaddap not found. Make sure it is in your MATLAB path.'];
    err=1;
    return
end

%% URL Indices (Latitude, Longitude).
[latitude,longitude,iLAT,iLON]=latlon_seawind(LAT1,LON1,LAT2,LON2,LATINC,LONINC);     %This function generates latitude/longitude vectors
                                                                                    %and returns indices for call using opendap.
%% Define constraints.
C1=['[' num2str(iLAT(1)) ':' num2str(LATINC) ':' num2str(iLAT(2)) ']']; % ConstraintLatitude
C2=['[' num2str(iLON(1)) ':' num2str(LONINC) ':' num2str(iLON(2)) ']']; % ConstraintLongitude
if (asc && ~des)
    C3='[0]';
elseif (~asc && des)
    C3='[1]';
else
    C3='[0:1:1]';
end

%% Define filenames.
filenames1=[];
if (DataType1)
    filenames1=adeos_get_filenames_wind(DATE1,DATE2,DATEINC); %This function determines filenames.
end
filenames2=[];
if (DataType2)
    filenames2=adeos_get_filenames_stress(DATE1,DATE2,DATEINC); %This function determines filenames.
end

% Handle errors.--> cannot connect to catalog.
err1=0;
if (DataType1 && isempty(filenames1))
    err1=1;
end
err2=0;
if (DataType2 && isempty(filenames2))
    err2=1;
end
if (DataType1 && ~DataType2 && err1)
    msg=['The number of files for "Wind Vectors" is zero or',...
        ' a connection could not be made. Please try again at a future date.'];
    err=1;
    data=[];
    return
end
if (DataType2 && ~DataType1 && err2)
    msg=['The number of files for "Wind Stress" is zero or',...
        ' a connection could not be made. Please try again at a future date.'];
    err=1;
    data=[];
    return
end
if (DataType1 && DataType2 && err1 && err2)
    msg=['The number of files for "Wind Vectors" and "Wind Stress" is zero or',...
        ' a connection could not be made. Please try again at a future date.'];
    err=1;
    data=[];
    return
end

% Estimate amount of data.
nfile=size(filenames1,1);
npass=asc+des; %1 if just asc or des, 2 if both. Number of passes.
nvariables = sum(Field.value(1:8)); %number of variables.
nlat=length(latitude);
nlon=length(longitude);
nbyt1=8*nlat*nlon*nfile*npass*nvariables;

% Estimate amount of data.
nfile=size(filenames2,1);
npass=asc+des; %1 if just asc or des, 2 if both. Number of passes.
nvariables = sum(Field.value(9:16)); %number of variables.
nlat=length(latitude);
nlon=length(longitude);
nbyt2=8*nlat*nlon*nfile*npass*nvariables;

if (nbyt1 + nbyt2 > memory_limit)
    msg=['The amount of required memory exceeds the specified memory limit of ',...
        num2str(MBLIMIT),' MB. What would you like to do?'];
    ButtonName=questdlg(msg, ...
                           'Memory Limit Exceeded', ...
                           'Continue','Stop','Continue');
    switch ButtonName,
        case 'Continue',
            
        case 'Stop',
            msg=['You have chosen to stop execution of the program.'];
            err=1;
            data=[];
            return
    end % switch
    
    pause(.1);
end

%% Ascending and Descending time.
time_frc_asc = 0.9375;
time_frc_des = 0.4375;

%% Get opendap structure number.
count= op_getnum('opendap','base');
count = count+1;

%% Attributes.
nfile1=size(filenames1,1);
nfile2=size(filenames2,1);
if (~switch_scale_each)
    if (DataType1 && nfile1>0)
    url = filenames1(1,:); %use first file as template
    attr1 = loaddap('-A',url); % get attributes
    end
    if (DataType2 && nfile2>0)
    url = filenames2(1,:); %use first file as template
    attr2 = loaddap('-A',url); % get attributes
    end
end%attributes

readme = '';

%% DataType1 --> Wind Vectors.
nfile1=size(filenames1,1);
if (DataType1 && nfile1>0)
    % Loop.
    name='adeos';
    for iK=1:nfile1
        npass = length(Pass.value);
        for iL=1:npass %ascending,descending
        
        if (~Pass.value(iL))
            continue %skip the following code.
        else %if statement over entire read/write code.
        
        if (iL == 1)
            time_frc = time_frc_asc;
            C3='[0]';
        end
        if (iL == 2)
            time_frc = time_frc_des;
            C3='[1]';
        end
        
        url=filenames1(iK,:);
        CE=[];
        if (~switch_scale_each)
            attr = attr1;
        else%Attributes
            attr=loaddap('-A',url); % get attributes
        end%Attributes
        
        % Load data using url protocol.
        if (Field.value(1))
            CE=[CE,'rep_wind_speed',C1,C2,C3];
            CE=[CE,','];
        end
        if (Field.value(2))
            CE=[CE,'rep_wind_velocity_u',C1,C2,C3];
            CE=[CE,','];
        end
        if (Field.value(3))
            CE=[CE,'rep_wind_velocity_v',C1,C2,C3];
            CE=[CE,','];
        end
        if (Field.value(4))
            CE=[CE,'rep_atten_corr',C1,C2,C3];
            CE=[CE,','];
        end
        if (Field.value(5))
            CE=[CE,'rep_time_of_day',C1,C2,C3];
            CE=[CE,','];
        end
        if (Field.value(6))
            CE=[CE,'rep_rain_probability',C1,C2,C3];
            CE=[CE,','];
        end
        if (Field.value(7))
            CE=[CE,'rep_srad_rain_rate',C1,C2,C3];
            CE=[CE,','];
        end
        if (Field.value(8))
            CE=[CE,'rep_amsr_rain_indicator',C1,C2,C3];
            CE=[CE,','];
        end
        if (Field.value(9))
            CE=[CE,'rain_flag',C1,C2,C3];
            CE=[CE,','];
        end
        if (Field.value(10))
            CE=[CE,'null_data_indicator',C1,C2,C3];
            CE=[CE,','];
        end
        if (Field.value(11))
            CE=[CE,'grid_cell_quality_flag',C1,C2,C3];
            CE=[CE,','];
        end
        len=length(CE);
        CE=CE(1:len-1); %remove comma.

        % Load data.
        disp([url,'?',CE])
        if (~native)
            tmp = loaddap([url,'?',CE]);
        else%native
            data_tmp = loaddap([url,'?',CE]);
        end%native

        % Scale data.
        if (~native)
        if (Field.value(1)),
            data_tmp.wind_sp=tmp.rep_wind_speed.*attr.rep_wind_speed.scale_factor;
%             readme_tmp = ['Scale: multiplied "rep_wind_speed" by ',...
%                     num2str(attr.rep_wind_speed.scale_factor)];
%             readme = [readme,'/',readme_tmp];
        end
        if (Field.value(2)),
            data_tmp.wind_u=tmp.rep_wind_velocity_u.*attr.rep_wind_velocity_u.scale_factor;
%             readme_tmp = ['Scale: multiplied "rep_wind_velocity_u" by ',...
%                     num2str(attr.rep_wind_velocity_u.scale_factor)];
%             readme = [readme,'/',readme_tmp];
        end
        if (Field.value(3)),
            data_tmp.wind_v=tmp.rep_wind_velocity_v.*attr.rep_wind_velocity_v.scale_factor;
%             readme_tmp = ['Scale: multiplied "rep_wind_velocity_v" by ',...
%                     num2str(attr.rep_wind_velocity_v.scale_factor)];
%             readme = [readme,'/',readme_tmp];
        end
        if (Field.value(4)),
            data_tmp.attn_corr=tmp.rep_atten_corr.*attr.rep_atten_corr.scale_factor;
%             readme_tmp = ['Scale: multiplied "rep_atten_corr" by ',...
%                     num2str(attr.rep_atten_corr.scale_factor)];
%             readme = [readme,'/',readme_tmp];
        end
        if (Field.value(5)),
            data_tmp.time_frac=tmp.rep_time_of_day.*attr.rep_time_of_day.scale_factor;
%             readme_tmp = ['Scale: multiplied "rep_time_of_day" by ',...
%                     num2str(attr.rep_time_of_day.scale_factor)];
%             readme = [readme,'/',readme_tmp];
        end
        if (Field.value(6)),
            data_tmp.rain_prob=tmp.rep_rain_probability.*attr.rep_rain_probability.scale_factor;
%             readme_tmp = ['Scale: multiplied "rep_rain_probability" by ',...
%                     num2str(attr.rep_rain_probability.scale_factor)];
%             readme = [readme,'/',readme_tmp];
        end
        if (Field.value(7)),
            data_tmp.rain_rate=tmp.rep_srad_rain_rate.*attr.rep_srad_rain_rate.scale_factor;
%             readme_tmp = ['Scale: multiplied "rep_srad_rain_rate" by ',...
%                     num2str(attr.rep_srad_rain_rate.scale_factor)];
%             readme = [readme,'/',readme_tmp];
        end
        if (Field.value(8)),
            data_tmp.amsr_rain=tmp.rep_amsr_rain_indicator.*attr.rep_amsr_rain_indicator.scale_factor;
%             readme_tmp = ['Scale: multiplied "rep_amsr_rain_indicator" by ',...
%                     num2str(attr.rep_amsr_rain_indicator.scale_factor)];
%             readme = [readme,'/',readme_tmp];
        end
        if (Field.value(9)),
            data_tmp.rain_flag=tmp.rain_flag.*attr.rain_flag.scale_factor;
%             readme_tmp = ['Scale: multiplied "rain_flag" by ',...
%                     num2str(attr.rain_flag.scale_factor)];
%             readme = [readme,'/',readme_tmp];
        end
        if (Field.value(10)),
            data_tmp.null_data=tmp.null_data_indicator.*attr.null_data_indicator.scale_factor;
%             readme_tmp = ['Scale: multiplied "null_data_indicator" by ',...
%                     num2str(attr.null_data_indicator.scale_factor)];
%             readme = [readme,'/',readme_tmp];
        end
        if (Field.value(11)),
            data_tmp.cell_qual=tmp.grid_cell_quality_flag.*attr.grid_cell_quality_flag.scale_factor;
%             readme_tmp = ['Scale: multiplied "grid_cell_quality_flag" by ',...
%                     num2str(attr.grid_cell_quality_flag.scale_factor)];
%             readme = [readme,'/',readme_tmp];
        end
        end%native
        
        %[2 1 3] --> pass      x longitude x latitude
        %[1 3 2] --> longitude x latitude  x pass
        %[2 3 1] --> pass      x latitude  x longitude

        if (exist('data_tmp','var'))
            fields=fieldnames(data_tmp);
            order=[3 1 2];
            data_tmp=permute_struct(data_tmp,fields,order);
            err=0;
            msg='';
        else
            data=[];
            err=1;
            msg=['Either no selections were made as to the variables OR',...
                ' there is something wrong with "loaddap". "Wind Vector" data have not been obtained.'];
            return
        end
        
        % time, date.
        iloc=strfind(url,'SW_S3S_');
        yyyyddd=url(iloc+7:iloc+13);
        gtime = op_greg2(str2double(yyyyddd(5:7))-1 + time_frc,str2double(yyyyddd(1:4)));
        %op_greg2 starts at yearday = 0.0.
        time = op_epoch(gtime);
        readme_tmp = ['time in seconds since reference time; '];
                readme = [readme,readme_tmp];
        readme_tmp = ['time in sun-synchronous time; '];
                readme = [readme,readme_tmp];
        %readme_tmp = ['missing values replaced with NaN; '];
        %        readme = [readme,readme_tmp];
        variable_name=['opendap','_',num2str(count,'%04d')];
        yyyymmdd=[num2str(gtime(1),'%04d'),...
                  num2str(gtime(2),'%02d'),...
                  num2str(gtime(3),'%02d')];
        if (native)
            if (iL==1)
                pass_txt = 'a';
            else
                pass_txt = 'd';
            end
            variable_name=[name,'_',yyyymmdd,pass_txt];
        end
        count = count+1;

        % Dependent variables.
        % Independent variables
        % Metadata.
        if (~native)
        
        % Dependent variables..
        depend_fields = fieldnames(data_tmp);
        
        % Independent variables.
        data_tmp.latitude=latitude;
        data_tmp.longitude=longitude;
        data_tmp.time=time;
        %data_tmp.depth=nan;
        
        % Metadata.
        tt = sprintf('%s',''''); %SINGLE QUOTE MARK
        str = ['{',tt,'latitude',tt,...
              ',',tt,'longitude',tt,...
              '}'];
        for idep=1:length(depend_fields)
            tmptmp = depend_fields{idep};
            eval(['data_tmp.metadata','.',tmptmp,'_variables',...
                 '=',str,';']);
        end
        data_tmp.metadata.url=[url,'?',CE]; %data_tmp.ce=['?',CE];
        if (Attributes)
            data_tmp.metadata.attributes=attr;
        end%attributes
        data_tmp.metadata.name=name;
        data_tmp.metadata.temporal = temporal;
        data_tmp.metadata.reference_time = datestr([1970,01,01,00,00,00],31);
        data_tmp.metadata.readme = readme; readme = '';
        data_tmp.metadata.user_friendly_time = datestr(gtime,31);
        %data_tmp.request_id = request_id1;
        data_tmp.metadata.request_date = request_date1;
        data_tmp.metadata.request = request;
        end

        % Put data in base workspace.
        if (SaveWorkspace && fmt==0)
            ws='base';
            assignin(ws,variable_name, data_tmp);
            %disp(['Writing variable "',variable_name,'" to current workspace.']);
        end %SaveWorkspace

        % Write ".mat" file.
        if (SaveFile && fmt==0 && strcmpi(SaveFileType,'.mat'))
            eval([variable_name,'=','data_tmp;']);
            slash='/';
            %if (isempty(prefx)), prefx=[name,'_']; end
            if (isempty(directory)), directory=pwd; end
            if (ispc), slash='\'; end
            if (isunix), slash='/'; end
            %if (strcmp(computer,'mac') || strcmp(computer,'maci')), slash='/'; end
            fname=[directory,slash,prefx,yyyymmdd,'.mat'];
            save(fname,variable_name);
            disp(['Writing variables to file ... ',fname]);
            eval(['clear ',variable_name]);
        end %.mat
        
%         % Pass data up to user.
%         if (fmt==1)
%             eval(['data.',variable_name,'=','data_tmp;']);
%             disp(['Placing variables in larger structure ... ',variable_name]);
%             eval(['clear ',variable_name]);
%         end %SaveWorkspace

        end%if statement over entire read/write code.

        end %loop over orbit    
    end %loop over files
end %DataType1.

if (exist('tmp','var')), clear tmp; end
if (exist('data_tmp','var')), clear data_tmp; end

%% DataType2 --> Wind Stress.
nfile2=size(filenames2,1);
if (DataType2 && nfile2>0)
    % Loop.
    name='adeos_stress';
    for iK=1:nfile2
        npass = length(Pass.value);
        for iL=1:npass %ascending,descending

        if (~Pass.value(iL))
            continue %skip the following code.
        else %if statement over entire read/write code.
            
        url=filenames2(iK,:);
        CE=[];
        if (~switch_scale_each)
            attr = attr2;
        else%Attributes
            attr=loaddap('-A',url); % get attributes
        end%Attributes
        
        if (iL == 1) %ascending
            
            time_frc = time_frc_asc;

            % Load data using url protocol.
            if (Field.value(12))
                CE=[CE,'asc_stress_Large_U',C1,C2,','];
            end
            if (Field.value(13))
                CE=[CE,'asc_stress_Large_V',C1,C2,','];
            end
            if (Field.value(14))
                CE=[CE,'asc_stress_Liu_U',C1,C2,','];
            end
            if (Field.value(15))
                CE=[CE,'asc_stress_Liu_V',C1,C2,','];
            end
            if (Field.value(16))
                CE=[CE,'asc_cd_Large',C1,C2,','];
            end
            if (Field.value(17))
                CE=[CE,'asc_cd_Liu',C1,C2,','];
            end
            if (Field.value(18))
                CE=[CE,'asc_rain_flag',C1,C2,','];
            end
            if (Field.value(19))
                CE=[CE,'asc_time_frac',C1,C2,','];
            end
            len=length(CE);
            CE=CE(1:len-1); %remove comma.
            
            % Load data.
            disp([url,'?',CE])
            if (~native)
                loaddap([url,'?',CE]);
            else%native
                data_tmp = loaddap([url,'?',CE]);
            end%native

            % Scale data.
            if (~native)
            K=1;
            if (exist('asc_cd_Large','var')),
                cd_Large_mat(K,:,:)= ...
                    asc_cd_Large.*attr.asc_cd_Large.scale_factor;
%                 readme_tmp = ['Scale: multiplied "asc_cd_Large" by ',...
%                     num2str(attr.asc_cd_Large.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('asc_cd_Liu','var')),
                cd_Liu_mat(K,:,:)= ...
                    asc_cd_Liu.*attr.asc_cd_Liu.scale_factor;
%                 readme_tmp = ['Scale: multiplied "asc_cd_Liu" by ',...
%                     num2str(attr.asc_cd_Liu.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('asc_rain_flag','var')),
                rain_flag_mat(K,:,:)= ...
                    asc_rain_flag.*attr.asc_rain_flag.scale_factor;
%                 readme_tmp = ['Scale: multiplied "asc_rain_flag" by ',...
%                     num2str(attr.asc_rain_flag.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('asc_stress_Large_U','var')),
                stress_Large_U_mat(K,:,:)= ...
                    asc_stress_Large_U.*attr.asc_stress_Large_U.scale_factor;
%                 readme_tmp = ['Scale: multiplied "asc_stress_Large_U" by ',...
%                     num2str(attr.asc_stress_Large_U.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('asc_stress_Large_V','var')),
                stress_Large_V_mat(K,:,:)= ...
                    asc_stress_Large_V.*attr.asc_stress_Large_V.scale_factor;
%                 readme_tmp = ['Scale: multiplied "asc_stress_Large_V" by ',...
%                     num2str(attr.asc_stress_Large_V.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('asc_stress_Liu_U','var')),
                stress_Liu_U_mat(K,:,:)= ...
                    asc_stress_Liu_U.*attr.asc_stress_Liu_U.scale_factor;
%                 readme_tmp = ['Scale: multiplied "asc_stress_Liu_U" by ',...
%                     num2str(attr.asc_stress_Liu_U.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('asc_stress_Liu_V','var')),
                stress_Liu_V_mat(K,:,:)= ...
                    asc_stress_Liu_V.*attr.asc_stress_Liu_V.scale_factor;
%                 readme_tmp = ['Scale: multiplied "asc_stress_Liu_V" by ',...
%                     num2str(attr.asc_stress_Liu_V.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('asc_time_frac','var')),
                time_frac_mat(K,:,:)= ...
                    asc_time_frac.*attr.asc_time_frac.scale_factor;
%                 readme_tmp = ['Scale: multiplied "asc_time_frac" by ',...
%                     num2str(attr.asc_time_frac.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            
            end%native
        end %asc
        
        if (iL == 2) %descending
            
            time_frc = time_frc_des;
            
            if (Field.value(12))
                CE=[CE,'des_stress_Large_U',C1,C2,','];
            end
            if (Field.value(13))
                CE=[CE,'des_stress_Large_V',C1,C2,','];
            end
            if (Field.value(14))
                CE=[CE,'des_stress_Liu_U',C1,C2,','];
            end
            if (Field.value(15))
                CE=[CE,'des_stress_Liu_V',C1,C2,','];
            end
            if (Field.value(16))
                CE=[CE,'des_cd_Large',C1,C2,','];
            end
            if (Field.value(17))
                CE=[CE,'des_cd_Liu',C1,C2,','];
            end
            if (Field.value(18) || Field.value(15))
                CE=[CE,'des_rain_flag',C1,C2,','];
            end
            if (Field.value(19) || Field.value(16))
                CE=[CE,'des_time_frac',C1,C2,','];
            end
            len=length(CE);
            CE=CE(1:len-1); %remove comma.
            
            % Load data.
            disp([url,'?',CE])
            if (~native)
                loaddap([url,'?',CE]);
            else%native
                data_tmp = loaddap([url,'?',CE]);
            end%native

            % Scale data.
            if (~native)
            K=1;
            if (exist('des_cd_Large','var')),
                cd_Large_mat(K,:,:)= ...
                    des_cd_Large.*attr.des_cd_Large.scale_factor;
%                 readme_tmp = ['Scale: multiplied "des_cd_Large" by ',...
%                     num2str(attr.des_cd_Large.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('des_cd_Liu','var')),
                cd_Liu_mat(K,:,:)= ...
                    des_cd_Liu.*attr.des_cd_Liu.scale_factor;
%                 readme_tmp = ['Scale: multiplied "des_cd_Liu" by ',...
%                     num2str(attr.des_cd_Liu.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('des_rain_flag','var')),
                rain_flag_mat(K,:,:)= ...
                    des_rain_flag.*attr.des_rain_flag.scale_factor;
%                 readme_tmp = ['Scale: multiplied "des_rain_flag" by ',...
%                     num2str(attr.des_rain_flag.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('des_stress_Large_U','var')),
                stress_Large_U_mat(K,:,:)= ...
                    des_stress_Large_U.*attr.des_stress_Large_U.scale_factor;
%                 readme_tmp = ['Scale: multiplied "des_stress_Large_U" by ',...
%                     num2str(attr.des_stress_Large_U.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('des_stress_Large_V','var')),
                stress_Large_V_mat(K,:,:)= ...
                    des_stress_Large_V.*attr.des_stress_Large_V.scale_factor;
%                 readme_tmp = ['Scale: multiplied "des_stress_Large_V" by ',...
%                     num2str(attr.des_stress_Large_V.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('des_stress_Liu_U','var')),
                stress_Liu_U_mat(K,:,:)= ...
                    des_stress_Liu_U.*attr.des_stress_Liu_U.scale_factor;
%                 readme_tmp = ['Scale: multiplied "des_stress_Liu_U" by ',...
%                     num2str(attr.des_stress_Liu_U.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('des_stress_Liu_V','var')),
                stress_Liu_V_mat(K,:,:)= ...
                    des_stress_Liu_V.*attr.des_stress_Liu_V.scale_factor;
%                 readme_tmp = ['Scale: multiplied "des_stress_Liu_V" by ',...
%                     num2str(attr.des_stress_Liu_V.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            if (exist('des_time_frac','var')),time_frac_mat(K,:,:)= ...
                    des_time_frac.*attr.des_time_frac.scale_factor;
%                 readme_tmp = ['Scale: multiplied "des_time_frac" by ',...
%                     num2str(attr.des_time_frac.scale_factor)];
%                 readme = [readme,'/',readme_tmp];
            end
            
            end%native
        end %des

        % Save data in structure.
        if (~native)
        if (exist('stress_Large_U_mat','var')),     data_tmp.stress_large_u=squeeze(stress_Large_U_mat); end
        if (exist('stress_Large_V_mat','var')),     data_tmp.stress_large_v=squeeze(stress_Large_V_mat); end
        if (exist('stress_Liu_U_mat','var')),       data_tmp.stress_liu_u=squeeze(stress_Liu_U_mat); end
        if (exist('stress_Liu_V_mat','var')),       data_tmp.stress_liu_v=squeeze(stress_Liu_V_mat); end
        if (exist('cd_Large_mat','var')),           data_tmp.cd_large=squeeze(cd_Large_mat); end
        if (exist('cd_Liu_mat','var')),             data_tmp.cd_liu=squeeze(cd_Liu_mat); end
        if (exist('rain_flag_mat','var')),          data_tmp.rain_flag=squeeze(rain_flag_mat); end
        if (exist('time_frac_mat','var')),          data_tmp.time_frac=squeeze(time_frac_mat); end
        end%native

        if (exist('data_tmp','var'))
            err=0;
            msg='';
        else
            data=[];
            err=1;
            msg=['Either no selections were made as to the variables OR',...
                ' there is something wrong with "loaddap". "Wind Stress" data have not been obtained.'];
            return
        end

        % time, date.
        iloc=strfind(url,'SW_ST3_');
        yyyyddd=url(iloc+7:iloc+13);
        gtime = op_greg2(str2double(yyyyddd(5:7))-1 + time_frc,str2double(yyyyddd(1:4)));
        %op_greg2 starts at yearday = 0.0.
        time = op_epoch(gtime);
        readme_tmp = ['time in seconds since reference time; '];
                readme = [readme,readme_tmp];
        readme_tmp = ['time in sun-synchronous time; '];
                readme = [readme,readme_tmp];
        %readme_tmp = ['missing values replaced with NaN; '];
        %        readme = [readme,readme_tmp];
        variable_name=['opendap','_',num2str(count,'%04d')];
        yyyymmdd=[num2str(gtime(1),'%04d'),...
                  num2str(gtime(2),'%02d'),...
                  num2str(gtime(3),'%02d')];
        if (native)
            if (iL==1)
                pass_txt = 'a';
            else
                pass_txt = 'd';
            end
            variable_name=[name,'_',yyyymmdd,pass_txt];
        end
        count = count+1;

        % Dependent variables.
        % Independent variables
        % Metadata.
        if (~native)
        
        % Dependent variables..
        depend_fields = fieldnames(data_tmp);
        
        % Independent variables.
        data_tmp.latitude=latitude;
        data_tmp.longitude=longitude;
        data_tmp.time=time;
        %data_tmp.depth=nan;
        
        % Metadata.
        tt = sprintf('%s',''''); %SINGLE QUOTE MARK
        str = ['{',tt,'latitude',tt,...
              ',',tt,'longitude',tt,...
              '}'];
        for idep=1:length(depend_fields)
            tmptmp = depend_fields{idep};
            eval(['data_tmp.metadata','.',tmptmp,'_variables',...
                 '=',str,';']);
        end
        data_tmp.metadata.url=[url,'?',CE]; %data_tmp.ce=['?',CE];
        if (Attributes)
            data_tmp.metadata.attributes=attr;
        end%attributes
        data_tmp.metadata.name=name;
        data_tmp.metadata.temporal = temporal;
        data_tmp.metadata.reference_time = datestr([1970,01,01,00,00,00],31);
        data_tmp.metadata.readme = readme; readme = '';
        data_tmp.metadata.user_friendly_time = datestr(gtime,31);
        %data_tmp.request_id = request_id1;
        data_tmp.metadata.request_date = request_date1;
        data_tmp.metadata.request = request;
        end

        % Put data in base workspace.
        if (SaveWorkspace && fmt==0)
            ws='base';
            assignin(ws,variable_name,data_tmp);
            %disp(['Writing variable "',variable_name,'" to current workspace.']);
        end %SaveWorkspace

        % Write ".mat" file.
        if (SaveFile && fmt==0 && strcmpi(SaveFileType,'.mat'))
            eval([variable_name,'=','data_tmp;']);
            slash='/';
            %if (isempty(prefx)), prefx=[name,'_']; end
            if (isempty(directory)), directory=pwd; end
            if (ispc), slash='\'; end
            if (isunix), slash='/'; end
            %if (strcmp(computer,'mac') || strcmp(computer,'maci')), slash='/'; end
            fname=[directory,slash,prefx,yyyymmdd,'.mat'];
            save(fname,variable_name);
            disp(['Writing variables to file ... ',fname]);
            eval(['clear ',variable_name]);
        end %.mat
        
%         % Pass data up to user.
%         if (fmt==1)
%             eval(['data.',variable_name,'=','data_tmp;']);
%             disp(['Placing variables in larger structure ... ',variable_name]);
%             eval(['clear ',variable_name]);
%         end %SaveWorkspace
        
        end%if statement over entire read/write code.
    
        end %loop over orbit
    end %loop over files
end %DataType2.

msg='';
err=0;
if ((nfile1==0 && nfile2==0) || fmt==0)
    data=[];
end
%data=[];    %change this if one wants to pass up the data through the functions.
            %If you want to do this, place DataType1 and DataType2 in a
            %single variable called "data", otherwise only wind stress will
            %be brought up to the top.

disp(['The data have been acquired.']);
return %endoffunction
%==========================================================================
function filenames=qscat_get_filenames_wind(DATE1,DATE2,DATEINC)
% This function determines filenames/urls by calling the catalogs assoc.
% ated with the data product.
%
% Christian Buckingham
% OPeNDAP 2006

file = 'get_seawinds_info.txt';
fid = fopen(file);
dummy = textscan(fid,'%s');
fclose(fid);
tmp = dummy{1};
ncol = 2;
nrow = length(tmp)/ncol;
tmp = reshape(tmp,[ncol,nrow])';
irow = 5;
icol = 2;
url_catalog = tmp{irow,icol};
irow = 9;
icol = 2;
instrument_name = tmp{irow,icol};

% instrument_name='QuikSCAT_L3';
% url_catalog='http://dods.jpl.nasa.gov/dods-bin/nph-dods/catalogs/quikscat/L3/quikscat_L3.dat';

url_projection2='DODS_URL';
date1=[DATE1(1:4),'/',DATE1(5:6),'/',DATE1(7:8)];
date2=[DATE2(1:4),'/',DATE2(5:6),'/',DATE2(7:8)];
url_selection=['date(',date1,',',date2,')'];

loaddap('+v',[url_catalog,'?',url_projection2,'&',url_selection]);
if (~exist(instrument_name,'var'));
    disp('Wind vector data files could not be found.');
    filenames=[];
    return
end
filenames=eval([instrument_name,'.DODS_URL']);

if (~strcmp(DATEINC,'1'));
    dateinc=str2num(DATEINC);
    idx=[1:dateinc:size(filenames,1)];
    filenames=filenames(idx,:);
end
    
return
%==========================================================================
function filenames=qscat_get_filenames_stress(DATE1,DATE2,DATEINC)
% This function determines filenames/urls by calling the catalogs assoc.
% ated with the data product.
%
% Christian Buckingham
% OPeNDAP 2006

file = 'get_seawinds_info.txt';
fid = fopen(file);
dummy = textscan(fid,'%s');
fclose(fid);
tmp = dummy{1};
ncol = 2;
nrow = length(tmp)/ncol;
tmp = reshape(tmp,[ncol,nrow])';
irow = 6;
icol = 2;
url_catalog = tmp{irow,icol};
irow = 10;
icol = 2;
instrument_name = tmp{irow,icol};

% instrument_name='QuikSCAT_L3';
% url_catalog='http://dods.jpl.nasa.gov/dods-bin/nph-dods/catalogs/quikscat/L3/quikscat_L3_stress.dat';

url_projection2='DODS_URL';
date1=[DATE1(1:4),'/',DATE1(5:6),'/',DATE1(7:8)];
date2=[DATE2(1:4),'/',DATE2(5:6),'/',DATE2(7:8)];
url_selection=['date(',date1,',',date2,')'];

loaddap('+v',[url_catalog,'?',url_projection2,'&',url_selection]);
if (~exist(instrument_name,'var'));
    disp('Wind stress: No data files could be found.');
    filenames=[];
    return
end
filenames=eval([instrument_name,'.DODS_URL']);

if (~strcmp(DATEINC,'1'));
    dateinc=str2num(DATEINC);
    idx=[1:dateinc:size(filenames,1)];
    filenames=filenames(idx,:);
end
    
return
%==========================================================================
function filenames=adeos_get_filenames_wind(DATE1,DATE2,DATEINC)
% This function determines filenames/urls by calling the catalogs assoc.
% ated with the data product.
%
% Christian Buckingham
% Dan Holloway
% OPeNDAP 2006

file = 'get_seawinds_info.txt';
fid = fopen(file);
dummy = textscan(fid,'%s');
fclose(fid);
tmp = dummy{1};
ncol = 2;
nrow = length(tmp)/ncol;
tmp = reshape(tmp,[ncol,nrow])';
irow = 7;
icol = 2;
url_catalog = tmp{irow,icol};
irow = 11;
icol = 2;
instrument_name = tmp{irow,icol};

% instrument_name='seawinds_L3';
% url_catalog='http://dods.jpl.nasa.gov/dods-bin/nph-dods/catalogs/seawinds/L3/seawinds_L3.dat';

url_projection2='DODS_URL';
%url_projection3=['DODS_Date_Time(',instrument_name,')'];
date1=[DATE1(1:4),'/',DATE1(5:6),'/',DATE1(7:8)];
date2=[DATE2(1:4),'/',DATE2(5:6),'/',DATE2(7:8)];
url_selection=['date(',date1,',',date2,')'];

loaddap('+v',[url_catalog,'?',url_projection2,'&',url_selection]);
%loaddap('+v',[url_catalog,'?',url_projection1,',',url_projection2,',',url_projection3,'&',url_selection]);
if (~exist(instrument_name,'var'));
    disp('Wind vectors: No data files could be found.');
    filenames=[];
    return
end
filenames=eval([instrument_name,'.DODS_URL']);

if (~strcmp(DATEINC,'1'));
    dateinc=str2num(DATEINC);
    idx=[1:dateinc:size(filenames,1)];
    filenames=filenames(idx,:);
end
    
return
%==========================================================================
function filenames=adeos_get_filenames_stress(DATE1,DATE2,DATEINC)
% This function determines filenames/urls by calling the catalogs assoc.
% ated with the data product.
%
% Christian Buckingham
% OPeNDAP 2006

file = 'get_seawinds_info.txt';
fid = fopen(file);
dummy = textscan(fid,'%s');
fclose(fid);
tmp = dummy{1};
ncol = 2;
nrow = length(tmp)/ncol;
tmp = reshape(tmp,[ncol,nrow])';
irow = 8;
icol = 2;
url_catalog = tmp{irow,icol};
irow = 12;
icol = 2;
instrument_name = tmp{irow,icol};

% instrument_name='seawinds_L3';
% url_catalog='http://dods.jpl.nasa.gov/dods-bin/nph-dods/catalogs/seawinds/L3/seawinds_L3_stress.dat';

url_projection2='DODS_URL';
date1=[DATE1(1:4),'/',DATE1(5:6),'/',DATE1(7:8)];
date2=[DATE2(1:4),'/',DATE2(5:6),'/',DATE2(7:8)];
url_selection=['date("',date1,'","',date2,'")'];

loaddap('+v',[url_catalog,'?',url_projection2,'&',url_selection]);
if (~exist(instrument_name,'var'));
    disp('Wind stress: No data files could be found.');
    filenames=[];
    return
end
filenames=eval([instrument_name,'.DODS_URL']);

if (~strcmp(DATEINC,'1'));
    dateinc=str2num(DATEINC);
    idx=[1:dateinc:size(filenames,1)];
    filenames=filenames(idx,:);
end
    
return
%==========================================================================
function [lats,lons,iLAT,iLON]=latlon_seawind(LAT1,LON1,LAT2,LON2,LATINC,LONINC)
% 2008/02/18 modified to include a single latitude/single longitude grid pt

%
% A function needs to be written to piece together data over the
% data discontinuity. For now data must be grabbed in pieces
% under the normal notation of [-180, -0.25] and [0,180]. Code will
% work if LON1 and LON2 are either between [-180, -0.25] or
% [0,180].

% PRELIMINARIES.
if (LON1<=-180),
    LON1=-180+.125;
end
if (LON2>=360),
    LON2=360-.125;
end
LON1=mod(LON1,360);
LON2=mod(LON2,360);

% nlat=720;
% nlon=1440;
% dlat=180/nlat;
% dlon=360/nlon;
% Latitude=((-90 +dlat/2):dlat:(-dlat/2+90));
% Longitude=(0 + dlon/2:dlon:-dlon/2 + 360);
load('get_seawinds_latlon.mat');
%load('gui_seawinds_latlon.mat');

% Convert desired lat/lon to indices.
len1=length(latitude);
len2=length(longitude);
ilat = interp1(latitude,(1:len1),[LAT1,LAT2],'nearest'); %find.m may be faster...
ilon = interp1(longitude,(1:len2),[LON1,LON2],'nearest');
if (LAT1 <= -90)
    ilat(1)=1;
end
if (LAT2 >= 90)
    ilat(2)=len1;
end
if (LON1 <= 0)
    ilon(1)=1;
end
if (LON2 >= 360)
    ilon(2)=len2;
end
% The following is a patch 2008/02/18
if LON1==LON2 %2008/02/18
    ilon(2) = ilon(1); %2008/02/18
end %2008/02/18
iLAT1=ilat(1)-1;
iLAT2=ilat(2)-1;
iLON1=ilon(1)-1;
iLON2=ilon(2)-1;

iLAT=[iLAT1,iLAT2];
iLON=[iLON1,iLON2];

lats=latitude(ilat(1):LATINC:ilat(2));
lons=longitude(ilon(1):LONINC:ilon(2));

return
%==========================================================================
function B=permute_struct(A,fields,order)
% B=permute_struct(A,fields,order);
%
% This function permutes the fields in a structure as specified by the
% vector "order". See qscat_get_data for more information. There is
% an example where it's used.
%
% Christian Buckingham
% OPeNDAP Project, 2006

B=A;
%fields=fieldnames(A);
len=length(fields);
for ii=1:len
    tmp_field=fields{ii};
    tmp_data=getfield(A,tmp_field);
    tmp_data=permute(tmp_data,order);
    B=setfield(B,tmp_field,tmp_data);
end

return %endoffunction
%==========================================================================
function [msg,err,DATE1,DATE2]=qscat_get_dates
% This function determines dates available by calling the catalogs assoc.
% ated with the data product.
%
% DATE1 -- start date for data
% DATE2 -- end date for data
%
% Christian Buckingham
% OPeNDAP 2006

instrument_name='QuikSCAT_L3';

url_catalog='http://dods.jpl.nasa.gov/dods-bin/nph-dods/catalogs/quikscat/L3/quikscat_L3.dat';
url_projection2='year';
url_projection3='month';
url_projection4='day';

% DATE=datestr(now,29);
% date=strrep(DATE,'-','/');

loaddap('+v',[url_catalog,'?',url_projection2,',',url_projection2,',',url_projection3,...
    ',',url_projection4]);

if (~exist(instrument_name))
    msg = ['loaddap connection could not be made. Please try again at a ',...
        'later time or check the connection.'];
    err=1;
    DATE1=[];
    DATE2=[];
    return
end

yyyy=eval([instrument_name,'.year']);
mm=eval([instrument_name,'.month']);
dd=eval([instrument_name,'.day']);

len=length(yyyy);
Y1=num2str(yyyy(1));
M1=num2str(mm(1));
D1=num2str(dd(1));
Y2=num2str(yyyy(len));
M2=num2str(mm(len));
D2=num2str(dd(len));

if (length(M1)==1); M1=['0',M1]; end
if (length(D1)==1); D1=['0',D1]; end

if (length(M2)==1); M2=['0',M2]; end
if (length(D2)==1); D2=['0',D2]; end

DATE1=[Y1,M1,D1];
DATE2=[Y2,M2,D2];

msg='';
err=0;

if (length(DATE1)~=8 || length(DATE2)~=8)
    msg = 'error: problem with QuikSCAT catalog file.';
    err = 1;
    return
end

return
%==========================================================================
function [msg,err,DATE1,DATE2]=adeos_get_dates
% This function determines dates available by calling the catalogs assoc.
% ated with the data product.
%
% DATE1 -- start date for data
% DATE2 -- end date for data
%
% Christian Buckingham
% OPeNDAP 2006

instrument_name='seawinds_L3';

url_catalog='http://dods.jpl.nasa.gov/dods-bin/nph-dods/catalogs/seawinds/L3/seawinds_L3.dat';
url_projection2='year';
url_projection3='month';
url_projection4='day';

% DATE=datestr(now,29);
% date=strrep(DATE,'-','/');

loaddap('+v',[url_catalog,'?',url_projection2,',',url_projection2,',',url_projection3,...
    ',',url_projection4]);

if (~exist(instrument_name))
    msg = ['loaddap connection could not be made. Please try again at a ',...
        'later time or check the connection.'];
    err=1;
    DATE1=[];
    DATE2=[];
    return
end

yyyy=eval([instrument_name,'.year']);
mm=eval([instrument_name,'.month']);
dd=eval([instrument_name,'.day']);

len=length(yyyy);
Y1=num2str(yyyy(1));
M1=num2str(mm(1));
D1=num2str(dd(1));
Y2=num2str(yyyy(len));
M2=num2str(mm(len));
D2=num2str(dd(len));

if (length(M1)==1); M1=['0',M1]; end
if (length(D1)==1); D1=['0',D1]; end

if (length(M2)==1); M2=['0',M2]; end
if (length(D2)==1); D2=['0',D2]; end

DATE1=[Y1,M1,D1];
DATE2=[Y2,M2,D2];

msg='';
err=0;

if (length(DATE1)~=8 || length(DATE2)~=8)
    msg = 'error: problem with Adeos ii catalog file.';
    err = 1;
    return
end

return
%==========================================================================